@@ -3,63 +3,71 @@
 #include "core/messageobject.h"
 
 #include "3rd-party/boolinq/boolinq.h"
+#include "database/databasefactory.h"
 
 #include <QSqlDatabase>
 #include <QSqlError>
 #include <QSqlQuery>
 
-MessageObject::MessageObject(QSqlDatabase* db, const QString& feed_custom_id,
-                             int account_id, QList<Label*> available_labels,
-                             QObject* parent)
+MessageObject::MessageObject(QSqlDatabase* db, const QString& feed_custom_id, int account_id,
+                             const QList<Label*>& available_labels, bool is_new_message, QObject* parent)
   : QObject(parent), m_db(db), m_feedCustomId(feed_custom_id), m_accountId(account_id), m_message(nullptr),
-  m_availableLabels(available_labels) {}
+  m_availableLabels(available_labels), m_runningAfterFetching(is_new_message) {}
 
 void MessageObject::setMessage(Message* message) {
   m_message = message;
 }
 
+bool MessageObject::isDuplicate(DuplicationAttributeCheck attribute_check) const {
+  return isDuplicateWithAttribute(attribute_check);
+}
+
 bool MessageObject::isDuplicateWithAttribute(MessageObject::DuplicationAttributeCheck attribute_check) const {
   // Check database according to duplication attribute_check.
   QSqlQuery q(*m_db);
   QStringList where_clauses;
-  QList<QPair<QString, QVariant>> bind_values;
+  QVector<QPair<QString, QVariant>> bind_values;
 
   // Now we construct the query according to parameter.
   if ((attribute_check& DuplicationAttributeCheck::SameTitle) == DuplicationAttributeCheck::SameTitle) {
     where_clauses.append(QSL("title = :title"));
-    bind_values.append({ ":title", title() });
+    bind_values.append({ QSL(":title"), title() });
   }
 
   if ((attribute_check& DuplicationAttributeCheck::SameUrl) == DuplicationAttributeCheck::SameUrl) {
     where_clauses.append(QSL("url = :url"));
-    bind_values.append({ ":url", url() });
+    bind_values.append({ QSL(":url"), url() });
   }
 
   if ((attribute_check& DuplicationAttributeCheck::SameAuthor) == DuplicationAttributeCheck::SameAuthor) {
     where_clauses.append(QSL("author = :author"));
-    bind_values.append({ ":author", author() });
+    bind_values.append({ QSL(":author"), author() });
   }
 
   if ((attribute_check& DuplicationAttributeCheck::SameDateCreated) == DuplicationAttributeCheck::SameDateCreated) {
     where_clauses.append(QSL("date_created = :date_created"));
-    bind_values.append({ ":date_created", created().toMSecsSinceEpoch() });
+    bind_values.append({ QSL(":date_created"), created().toMSecsSinceEpoch() });
+  }
+
+  if ((attribute_check& DuplicationAttributeCheck::SameCustomId) == DuplicationAttributeCheck::SameCustomId) {
+    where_clauses.append(QSL("custom_id = :custom_id"));
+    bind_values.append({ QSL(":custom_id"), customId() });
   }
 
   where_clauses.append(QSL("account_id = :account_id"));
-  bind_values.append({ ":account_id", accountId() });
+  bind_values.append({ QSL(":account_id"), accountId() });
 
   if ((attribute_check& DuplicationAttributeCheck::AllFeedsSameAccount) != DuplicationAttributeCheck::AllFeedsSameAccount) {
     // Limit to current feed.
     where_clauses.append(QSL("feed = :feed"));
-    bind_values.append({ ":feed", feedCustomId() });
+    bind_values.append({ QSL(":feed"), feedCustomId() });
   }
 
   QString full_query = QSL("SELECT COUNT(*) FROM Messages WHERE ") + where_clauses.join(QSL(" AND ")) + QSL(";");
 
   qDebugNN << LOGSEC_MESSAGEMODEL
-           << "Query for MSG duplicate identification is: '"
-           << full_query
-           << "'.";
+           << "Prepared query for MSG duplicate identification is:"
+           << QUOTE_W_SPACE_DOT(full_query);
 
   q.setForwardOnly(true);
   q.prepare(full_query);
@@ -69,26 +77,33 @@ bool MessageObject::isDuplicateWithAttribute(MessageObject::DuplicationAttribute
   }
 
   if (q.exec() && q.next()) {
+    qDebugNN << LOGSEC_DB
+             << "Executed SQL for message duplicates check:"
+             << QUOTE_W_SPACE_DOT(DatabaseFactory::lastExecutedQuery(q));
+
     if (q.record().value(0).toInt() > 0) {
       // Whoops, we have the "same" message in database.
-      qDebugNN << LOGSEC_MESSAGEMODEL
-               << "Message '"
-               << title()
-               << "' was identified as duplicate by filter script.";
+      qDebugNN << LOGSEC_CORE
+               << "Message"
+               << QUOTE_W_SPACE(title())
+               << "was identified as duplicate by filter script.";
       return true;
     }
   }
   else if (q.lastError().isValid()) {
-    qWarningNN << LOGSEC_MESSAGEMODEL
-               << "Error when checking for duplicate messages via filtering system, error: '"
-               << q.lastError().text()
-               << "'.";
+    qWarningNN << LOGSEC_CORE
+               << "Error when checking for duplicate messages via filtering system, error:"
+               << QUOTE_W_SPACE_DOT(q.lastError().text());
   }
 
   return false;
 }
 
-bool MessageObject::assignLabel(QString label_custom_id) const {
+bool MessageObject::assignLabel(const QString& label_custom_id) const {
+  if (m_message->m_id <= 0 && m_message->m_customId.isEmpty()) {
+    return false;
+  }
+
   Label* lbl = boolinq::from(m_availableLabels).firstOrDefault([label_custom_id](Label* lbl) {
     return lbl->customId() == label_custom_id;
   });
@@ -105,7 +120,11 @@ bool MessageObject::assignLabel(QString label_custom_id) const {
   }
 }
 
-bool MessageObject::deassignLabel(QString label_custom_id) const {
+bool MessageObject::deassignLabel(const QString& label_custom_id) const {
+  if (m_message->m_id <= 0 && m_message->m_customId.isEmpty()) {
+    return false;
+  }
+
   Label* lbl = boolinq::from(m_message->m_assignedLabels).firstOrDefault([label_custom_id](Label* lbl) {
     return lbl->customId() == label_custom_id;
   });
@@ -151,6 +170,14 @@ void MessageObject::setContents(const QString& contents) {
   m_message->m_contents = contents;
 }
 
+QString MessageObject::rawContents() const {
+  return m_message->m_rawContents;
+}
+
+void MessageObject::setRawContents(const QString& raw_contents) {
+  m_message->m_rawContents = raw_contents;
+}
+
 QDateTime MessageObject::created() const {
   return m_message->m_created;
 }
@@ -183,6 +210,14 @@ void MessageObject::setIsDeleted(bool is_deleted) {
   m_message->m_isDeleted = is_deleted;
 }
 
+double MessageObject::score() const {
+  return m_message->m_score;
+}
+
+void MessageObject::setScore(double score) {
+  m_message->m_score = score;
+}
+
 QString MessageObject::feedCustomId() const {
   if (m_feedCustomId.isEmpty() || m_feedCustomId == QString::number(NO_PARENT_CATEGORY)) {
     return m_message->m_feedId;
@@ -196,6 +231,14 @@ int MessageObject::accountId() const {
   return m_accountId;
 }
 
+QString MessageObject::customId() const {
+  return m_message->m_customId;
+}
+
+int MessageObject::id() const {
+  return m_message->m_id;
+}
+
 QList<Label*> MessageObject::assignedLabels() const {
   return m_message->m_assignedLabels;
 }
@@ -204,6 +247,6 @@ QList<Label*> MessageObject::availableLabels() const {
   return m_availableLabels;
 }
 
-bool MessageObject::alreadyStoredInDb() const {
-  return m_message->m_id > 0;
+bool MessageObject::runningFilterWhenFetching() const {
+  return m_runningAfterFetching;
 }
