@@ -14,9 +14,7 @@ class DynamicShortcutsWidget : public QWidget {
   Q_OBJECT
 
   public:
-
-    // Constructors and destructors.
-    explicit DynamicShortcutsWidget(QWidget* parent = 0);
+    explicit DynamicShortcutsWidget(QWidget* parent = nullptr);
     virtual ~DynamicShortcutsWidget();
 
     // Updates shortcuts of all actions according to changes.
@@ -40,7 +38,6 @@ class DynamicShortcutsWidget : public QWidget {
 
   private:
     QGridLayout* m_layout;
-
     QList<ActionBinding> m_actionBindings;
 };
 
