@@ -7,7 +7,7 @@
 
 #include "ui_formdatabasecleanup.h"
 
-#include "miscellaneous/databasecleaner.h"
+#include "database/databasecleaner.h"
 
 class FormDatabaseCleanup : public QDialog {
   Q_OBJECT
@@ -17,8 +17,9 @@ class FormDatabaseCleanup : public QDialog {
     virtual ~FormDatabaseCleanup() = default;
 
   protected:
-    void closeEvent(QCloseEvent* event);
-    void keyPressEvent(QKeyEvent* event);
+    virtual void closeEvent(QCloseEvent* event);
+    virtual void hideEvent(QHideEvent* event);
+    virtual void keyPressEvent(QKeyEvent* event);
 
   private slots:
     void updateDaysSuffix(int number);
