@@ -2,6 +2,7 @@
 
 #include "gui/dialogs/formmain.h"
 
+#include "database/databasefactory.h"
 #include "definitions/definitions.h"
 #include "gui/dialogs/formabout.h"
 #include "gui/dialogs/formaddaccount.h"
@@ -11,18 +12,17 @@
 #include "gui/dialogs/formsettings.h"
 #include "gui/dialogs/formupdate.h"
 #include "gui/feedmessageviewer.h"
-#include "gui/feedstoolbar.h"
 #include "gui/feedsview.h"
 #include "gui/messagebox.h"
 #include "gui/messagepreviewer.h"
-#include "gui/messagestoolbar.h"
 #include "gui/messagesview.h"
-#include "gui/plaintoolbutton.h"
-#include "gui/statusbar.h"
+#include "gui/reusable/plaintoolbutton.h"
 #include "gui/systemtrayicon.h"
 #include "gui/tabbar.h"
+#include "gui/toolbars/feedstoolbar.h"
+#include "gui/toolbars/messagestoolbar.h"
+#include "gui/toolbars/statusbar.h"
 #include "miscellaneous/application.h"
-#include "miscellaneous/databasefactory.h"
 #include "miscellaneous/feedreader.h"
 #include "miscellaneous/iconfactory.h"
 #include "miscellaneous/mutex.h"
@@ -31,10 +31,10 @@
 #include "network-web/webfactory.h"
 #include "services/abstract/recyclebin.h"
 #include "services/abstract/serviceroot.h"
-#include "services/owncloud/network/owncloudnetworkfactory.h"
+#include "services/owncloud/owncloudnetworkfactory.h"
 #include "services/standard/gui/formstandardimportexport.h"
 
-#if defined (USE_WEBENGINE)
+#if defined(USE_WEBENGINE)
 #include "network-web/adblock/adblockicon.h"
 #include "network-web/adblock/adblockmanager.h"
 #endif
@@ -60,9 +60,9 @@ FormMain::FormMain(QWidget* parent, Qt::WindowFlags f)
   qApp->setMainForm(this);
 
   setWindowIcon(qApp->desktopAwareIcon());
-  setWindowTitle(APP_LONG_NAME);
+  setWindowTitle(QSL(APP_LONG_NAME));
 
-#if defined (USE_WEBENGINE)
+#if defined(USE_WEBENGINE)
   m_ui->m_menuWebBrowserTabs->addAction(qApp->web()->adBlock()->adBlockIcon());
   m_ui->m_menuWebBrowserTabs->addAction(qApp->web()->engineSettingsAction());
 #endif
@@ -119,9 +119,11 @@ void FormMain::showDbCleanupAssistant() {
     qApp->feedReader()->feedsModel()->reloadCountsOfWholeModel();
   }
   else {
-    qApp->showGuiMessage(tr("Cannot cleanup database"),
+    qApp->showGuiMessage(Notification::Event::GeneralEvent,
+                         tr("Cannot cleanup database"),
                          tr("Cannot cleanup database, because another critical action is running."),
-                         QSystemTrayIcon::Warning, qApp->mainFormWidget(), true);
+                         QSystemTrayIcon::Warning,
+                         true);
   }
 }
 
@@ -190,6 +192,7 @@ QList<QAction*> FormMain::allActions() const {
   actions << m_ui->m_actionSelectPreviousMessage;
   actions << m_ui->m_actionSelectNextUnreadMessage;
   actions << m_ui->m_actionExpandCollapseItem;
+  actions << m_ui->m_actionExpandCollapseItemRecursively;
   actions << m_ui->m_actionMessageFilters;
 
 #if defined(USE_WEBENGINE)
@@ -204,16 +207,14 @@ QList<QAction*> FormMain::allActions() const {
 
 void FormMain::prepareMenus() {
   // Setup menu for tray icon.
-  if (SystemTrayIcon::isSystemTrayAvailable()) {
+  if (SystemTrayIcon::isSystemTrayAreaAvailable()) {
 #if defined(Q_OS_WIN)
-    m_trayMenu = new TrayIconMenu(APP_NAME, this);
+    m_trayMenu = new TrayIconMenu(QSL(APP_NAME), this);
 #else
     m_trayMenu = new QMenu(QSL(APP_NAME), this);
 #endif
 
     // Add needed items to the menu.
-    m_trayMenu->addAction(m_ui->m_actionSwitchMainWindow);
-    m_trayMenu->addSeparator();
     m_trayMenu->addAction(m_ui->m_actionUpdateAllItems);
     m_trayMenu->addAction(m_ui->m_actionMarkAllItemsRead);
     m_trayMenu->addSeparator();
@@ -225,12 +226,18 @@ void FormMain::prepareMenus() {
 
 #if !defined(USE_WEBENGINE)
   m_ui->m_menuWebBrowserTabs->removeAction(m_ui->m_actionTabNewWebBrowser);
-  m_ui->m_menuWebBrowserTabs->setTitle(tr("Tabs"));
+  m_ui->m_menuWebBrowserTabs->setTitle(tr("Ta&bs"));
 #endif
+
 #if defined(Q_OS_MACOS)
   m_ui->m_actionSwitchMainMenu->setVisible(false);
   m_ui->m_actionFullscreen->setVisible(false);
 #endif
+
+  if (QSysInfo::currentCpuArchitecture().contains(QSL("arm"), Qt::CaseSensitivity::CaseInsensitive)) {
+    qWarningNN << LOGSEC_GUI << "Disabling native menu bar.";
+    m_ui->m_menuBar->setNativeMenuBar(false);
+  }
 }
 
 void FormMain::switchFullscreenMode() {
@@ -252,7 +259,9 @@ void FormMain::updateAddItemMenu() {
   // NOTE: Clear here deletes items from memory but only those OWNED by the menu.
   m_ui->m_menuAddItem->clear();
 
-  for (ServiceRoot* activated_root : qApp->feedReader()->feedsModel()->serviceRoots()) {
+  auto srts = qApp->feedReader()->feedsModel()->serviceRoots();
+
+  for (ServiceRoot* activated_root : qAsConst(srts)) {
     QMenu* root_menu = new QMenu(activated_root->title(), m_ui->m_menuAddItem);
 
     root_menu->setIcon(activated_root->icon());
@@ -307,7 +316,9 @@ void FormMain::updateAddItemMenu() {
 void FormMain::updateRecycleBinMenu() {
   m_ui->m_menuRecycleBin->clear();
 
-  for (const ServiceRoot* activated_root : qApp->feedReader()->feedsModel()->serviceRoots()) {
+  auto srts = qApp->feedReader()->feedsModel()->serviceRoots();
+
+  for (const ServiceRoot* activated_root : qAsConst(srts)) {
     QMenu* root_menu = new QMenu(activated_root->title(), m_ui->m_menuRecycleBin);
 
     root_menu->setIcon(activated_root->icon());
@@ -349,7 +360,9 @@ void FormMain::updateRecycleBinMenu() {
 void FormMain::updateAccountsMenu() {
   m_ui->m_menuAccounts->clear();
 
-  for (ServiceRoot* activated_root : qApp->feedReader()->feedsModel()->serviceRoots()) {
+  auto srts = qApp->feedReader()->feedsModel()->serviceRoots();
+
+  for (ServiceRoot* activated_root : srts) {
     QMenu* root_menu = new QMenu(activated_root->title(), m_ui->m_menuAccounts);
 
     root_menu->setIcon(activated_root->icon());
@@ -389,14 +402,14 @@ void FormMain::onFeedUpdatesFinished(const FeedDownloadResults& results) {
 
 void FormMain::onFeedUpdatesStarted() {
   m_ui->m_actionStopRunningItemsUpdate->setEnabled(true);
-  statusBar()->showProgressFeeds(0, tr("Feed update started"));
+  statusBar()->showProgressFeeds(-1, tr("Fetching common data"));
 }
 
 void FormMain::onFeedUpdatesProgress(const Feed* feed, int current, int total) {
   statusBar()->showProgressFeeds(int((current * 100.0) / total),
 
                                  //: Text display in status bar when particular feed is updated.
-                                 tr("Updated feed '%1'").arg(feed->title()));
+                                 tr("Fetched '%1'").arg(TextFactory::shorten(feed->title())));
 }
 
 void FormMain::updateMessageButtonsAvailability() {
@@ -437,7 +450,8 @@ void FormMain::updateFeedButtonsAvailability() {
   m_ui->m_actionUpdateSelectedItemsWithCustomTimers->setEnabled(!critical_action_running);
   m_ui->m_actionUpdateSelectedItems->setEnabled(!critical_action_running && (feed_selected || category_selected || service_selected));
   m_ui->m_actionViewSelectedItemsNewspaperMode->setEnabled(anything_selected);
-  m_ui->m_actionExpandCollapseItem->setEnabled(anything_selected);
+  m_ui->m_actionExpandCollapseItem->setEnabled(category_selected || service_selected);
+  m_ui->m_actionExpandCollapseItemRecursively->setEnabled(category_selected || service_selected);
   m_ui->m_actionServiceDelete->setEnabled(service_selected);
   m_ui->m_actionServiceEdit->setEnabled(service_selected);
   m_ui->m_actionAddFeedIntoSelectedItem->setEnabled(anything_selected);
@@ -449,8 +463,17 @@ void FormMain::updateFeedButtonsAvailability() {
 
 void FormMain::switchVisibility(bool force_hide) {
   if (force_hide || isVisible()) {
-    if (SystemTrayIcon::isSystemTrayActivated()) {
-      hide();
+    if (SystemTrayIcon::isSystemTrayDesired() && SystemTrayIcon::isSystemTrayAreaAvailable()) {
+
+      if (QApplication::activeModalWidget() != nullptr) {
+        qApp->showGuiMessage(Notification::Event::GeneralEvent,
+                             QSL(APP_LONG_NAME),
+                             tr("Close opened modal dialogs first."),
+                             QSystemTrayIcon::Warning, true);
+      }
+      else {
+        hide();
+      }
     }
     else {
       // Window gets minimized in single-window mode.
@@ -534,6 +557,7 @@ void FormMain::setupIcons() {
   m_ui->m_actionShowOnlyUnreadItems->setIcon(icon_theme_factory->fromTheme(QSL("mail-mark-unread")));
   m_ui->m_actionShowOnlyUnreadMessages->setIcon(icon_theme_factory->fromTheme(QSL("mail-mark-unread")));
   m_ui->m_actionExpandCollapseItem->setIcon(icon_theme_factory->fromTheme(QSL("format-indent-more")));
+  m_ui->m_actionExpandCollapseItemRecursively->setIcon(icon_theme_factory->fromTheme(QSL("format-indent-more")));
   m_ui->m_actionRestoreSelectedMessages->setIcon(icon_theme_factory->fromTheme(QSL("view-refresh")));
   m_ui->m_actionRestoreAllRecycleBins->setIcon(icon_theme_factory->fromTheme(QSL("view-refresh")));
   m_ui->m_actionEmptyAllRecycleBins->setIcon(icon_theme_factory->fromTheme(QSL("edit-clear")));
@@ -576,7 +600,7 @@ void FormMain::loadSize() {
   move(settings->value(GROUP(GUI), GUI::MainWindowInitialPosition, screen.center() - rect().center()).toPoint());
 
   if (settings->value(GROUP(GUI), SETTING(GUI::MainWindowStartsMaximized)).toBool()) {
-    setWindowState(windowState() | Qt::WindowMaximized);
+    setWindowState(windowState() | Qt::WindowState::WindowMaximized);
 
     // We process events so that window is really maximized fast.
     qApp->processEvents();
@@ -730,7 +754,17 @@ void FormMain::createConnections() {
   connect(m_ui->m_actionMarkSelectedItemsAsRead,
           &QAction::triggered, tabWidget()->feedMessageViewer()->feedsView(), &FeedsView::markSelectedItemRead);
   connect(m_ui->m_actionExpandCollapseItem,
-          &QAction::triggered, tabWidget()->feedMessageViewer()->feedsView(), &FeedsView::expandCollapseCurrentItem);
+          &QAction::triggered,
+          tabWidget()->feedMessageViewer()->feedsView(),
+          [this]() {
+    tabWidget()->feedMessageViewer()->feedsView()->expandCollapseCurrentItem(false);
+  });
+  connect(m_ui->m_actionExpandCollapseItemRecursively,
+          &QAction::triggered,
+          tabWidget()->feedMessageViewer()->feedsView(),
+          [this]() {
+    tabWidget()->feedMessageViewer()->feedsView()->expandCollapseCurrentItem(true);
+  });
   connect(m_ui->m_actionMarkSelectedItemsAsUnread,
           &QAction::triggered, tabWidget()->feedMessageViewer()->feedsView(), &FeedsView::markSelectedItemUnread);
   connect(m_ui->m_actionClearSelectedItems,
@@ -770,7 +804,9 @@ void FormMain::createConnections() {
   connect(m_ui->m_actionSelectPreviousMessage,
           &QAction::triggered, tabWidget()->feedMessageViewer()->messagesView(), &MessagesView::selectPreviousItem);
   connect(m_ui->m_actionSwitchMessageListOrientation, &QAction::triggered,
-          tabWidget()->feedMessageViewer(), &FeedMessageViewer::switchMessageSplitterOrientation);
+          tabWidget()->feedMessageViewer(), [this]() {
+    tabWidget()->feedMessageViewer()->switchMessageSplitterOrientation(true);
+  });
   connect(m_ui->m_actionShowOnlyUnreadItems, &QAction::toggled,
           tabWidget()->feedMessageViewer(), &FeedMessageViewer::toggleShowOnlyUnreadFeeds);
   connect(m_ui->m_actionShowTreeBranches, &QAction::toggled,
@@ -811,9 +847,10 @@ void FormMain::restoreDatabaseSettings() {
 
 void FormMain::changeEvent(QEvent* event) {
   switch (event->type()) {
-    case QEvent::WindowStateChange: {
+    case QEvent::Type::WindowStateChange: {
       if ((windowState() & Qt::WindowState::WindowMinimized) == Qt::WindowState::WindowMinimized &&
-          SystemTrayIcon::isSystemTrayActivated() &&
+          SystemTrayIcon::isSystemTrayDesired() &&
+          SystemTrayIcon::isSystemTrayAreaAvailable() &&
           qApp->settings()->value(GROUP(GUI), SETTING(GUI::HideMainWindowWhenMinimized)).toBool()) {
         event->ignore();
         QTimer::singleShot(CHANGE_EVENT_DELAY, this, [this]() {
@@ -844,7 +881,7 @@ void FormMain::hideEvent(QHideEvent* event) {
 }
 
 void FormMain::showDocs() {
-  qApp->web()->openUrlInExternalBrowser(APP_URL_DOCUMENTATION);
+  qApp->web()->openUrlInExternalBrowser(QSL(APP_URL_DOCUMENTATION));
 }
 
 void FormMain::showAddAccountDialog() {
