@@ -6,10 +6,10 @@
 #include "core/messagesmodel.h"
 #include "core/messagesproxymodel.h"
 #include "gui/dialogs/formmain.h"
-#include "gui/labelsmenu.h"
 #include "gui/messagebox.h"
-#include "gui/styleditemdelegatewithoutfocus.h"
-#include "gui/treeviewcolumnsmenu.h"
+#include "gui/reusable/labelsmenu.h"
+#include "gui/reusable/styleditemdelegatewithoutfocus.h"
+#include "gui/reusable/treeviewcolumnsmenu.h"
 #include "miscellaneous/externaltool.h"
 #include "miscellaneous/feedreader.h"
 #include "miscellaneous/settings.h"
@@ -26,7 +26,8 @@
 #include <QTimer>
 #include <QTimer>
 
-MessagesView::MessagesView(QWidget* parent) : QTreeView(parent), m_contextMenu(nullptr), m_columnsAdjusted(false) {
+MessagesView::MessagesView(QWidget* parent)
+  : BaseTreeView(parent), m_contextMenu(nullptr), m_columnsAdjusted(false), m_processingMouse(false) {
   m_sourceModel = qApp->feedReader()->messagesModel();
   m_proxyModel = qApp->feedReader()->messagesProxyModel();
 
@@ -35,7 +36,7 @@ MessagesView::MessagesView(QWidget* parent) : QTreeView(parent), m_contextMenu(n
   setModel(m_proxyModel);
   setupAppearance();
   header()->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
-  connect(header(), &QHeaderView::customContextMenuRequested, this, [=](const QPoint& point) {
+  connect(header(), &QHeaderView::customContextMenuRequested, this, [=](QPoint point) {
     TreeViewColumnsMenu mm(header());
     mm.exec(header()->mapToGlobal(point));
   });
@@ -51,12 +52,71 @@ void MessagesView::reloadFontSettings() {
   m_sourceModel->setupFonts();
 }
 
-void MessagesView::sort(int column, Qt::SortOrder order, bool repopulate_data, bool change_header, bool emit_changed_from_header) {
+QByteArray MessagesView::saveHeaderState() const {
+  QByteArray arr;
+  QDataStream outt(&arr, QIODevice::OpenModeFlag::WriteOnly);
+
+  outt.setVersion(QDataStream::Version::Qt_4_7);
+  outt << header()->count();
+  outt << int(header()->sortIndicatorOrder());
+  outt << header()->sortIndicatorSection();
+
+  // Save column data.
+  for (int i = 0; i < header()->count(); i++) {
+    outt << header()->visualIndex(i);
+    outt << header()->sectionSize(i);
+    outt << header()->isSectionHidden(i);
+  }
+
+  return arr;
+}
+
+void MessagesView::restoreHeaderState(const QByteArray& dta) {
+  QByteArray arr = dta;
+  QDataStream inn(&arr, QIODevice::OpenModeFlag::ReadOnly);
+
+  inn.setVersion(QDataStream::Version::Qt_4_7);
+
+  int saved_header_count; inn >> saved_header_count;
+
+  if (std::abs(saved_header_count - header()->count()) > 10) {
+    qWarningNN << LOGSEC_GUI << "Detected invalid state for list view.";
+    return;
+  }
+
+  int saved_sort_order; inn >> saved_sort_order;
+  int saved_sort_column; inn >> saved_sort_column;
+
+  for (int i = 0; i < saved_header_count && i < header()->count(); i++) {
+    int vi, ss;
+    bool ish;
+
+    inn >> vi;
+    inn >> ss;
+    inn >> ish;
+
+    if (vi < header()->count()) {
+      header()->swapSections(header()->visualIndex(i), vi);
+    }
+
+    header()->resizeSection(i, ss);
+    header()->setSectionHidden(i, ish);
+  }
+
+  if (saved_sort_column < header()->count()) {
+    header()->setSortIndicator(saved_sort_column, Qt::SortOrder(saved_sort_order));
+  }
+}
+
+void MessagesView::sort(int column, Qt::SortOrder order,
+                        bool repopulate_data, bool change_header,
+                        bool emit_changed_from_header,
+                        bool ignore_multicolumn_sorting) {
   if (change_header && !emit_changed_from_header) {
     header()->blockSignals(true);
   }
 
-  m_sourceModel->addSortState(column, order);
+  m_sourceModel->addSortState(column, order, ignore_multicolumn_sorting);
 
   if (repopulate_data) {
     m_sourceModel->repopulate();
@@ -79,25 +139,29 @@ void MessagesView::createConnections() {
 void MessagesView::keyboardSearch(const QString& search) {
   // WARNING: This is quite hacky way how to force selection of next item even
   // with extended selection enabled.
-  setSelectionMode(QAbstractItemView::SingleSelection);
+  setSelectionMode(QAbstractItemView::SelectionMode::SingleSelection);
   QTreeView::keyboardSearch(search);
-  setSelectionMode(QAbstractItemView::ExtendedSelection);
+  setSelectionMode(QAbstractItemView::SelectionMode::ExtendedSelection);
 }
 
 void MessagesView::reloadSelections() {
   const QDateTime dt1 = QDateTime::currentDateTime();
   QModelIndex current_index = selectionModel()->currentIndex();
+  const bool is_current_selected = selectionModel()->selectedRows().contains(m_proxyModel->index(current_index.row(),
+                                                                                                 0,
+                                                                                                 current_index.parent()));
   const QModelIndex mapped_current_index = m_proxyModel->mapToSource(current_index);
   const Message selected_message = m_sourceModel->messageAt(mapped_current_index.row());
   const int col = header()->sortIndicatorSection();
   const Qt::SortOrder ord = header()->sortIndicatorOrder();
 
   // Reload the model now.
-  sort(col, ord, true, false, false);
+  sort(col, ord, true, false, false, true);
 
   // Now, we must find the same previously focused message.
   if (selected_message.m_id > 0) {
-    if (m_proxyModel->rowCount() == 0) {
+    if (m_proxyModel->rowCount() == 0 ||
+        !is_current_selected) {
       current_index = QModelIndex();
     }
     else {
@@ -153,6 +217,7 @@ void MessagesView::setupAppearance() {
   setItemDelegate(new StyledItemDelegateWithoutFocus(this));
   header()->setDefaultSectionSize(MESSAGES_VIEW_DEFAULT_COL);
   header()->setMinimumSectionSize(MESSAGES_VIEW_MINIMUM_COL);
+  header()->setFirstSectionMovable(true);
   header()->setCascadingSectionResizes(false);
   header()->setStretchLastSection(false);
 }
@@ -160,17 +225,27 @@ void MessagesView::setupAppearance() {
 void MessagesView::focusInEvent(QFocusEvent* event) {
   QTreeView::focusInEvent(event);
 
-  if (currentIndex().isValid()) {
+  qDebugNN << LOGSEC_GUI
+           << "Message list got focus with reason"
+           << QUOTE_W_SPACE_DOT(event->reason());
+
+  if ((event->reason()== Qt::FocusReason::TabFocusReason ||
+       event->reason()== Qt::FocusReason::BacktabFocusReason ||
+       event->reason()== Qt::FocusReason::ShortcutFocusReason) &&
+      currentIndex().isValid()) {
     selectionModel()->select(currentIndex(), QItemSelectionModel::SelectionFlag::Select | QItemSelectionModel::SelectionFlag::Rows);
   }
 }
 
 void MessagesView::keyPressEvent(QKeyEvent* event) {
-  QTreeView::keyPressEvent(event);
+  BaseTreeView::keyPressEvent(event);
 
   if (event->key() == Qt::Key::Key_Delete) {
     deleteSelectedMessages();
   }
+  else if (event->key() == Qt::Key::Key_Backspace) {
+    restoreSelectedMessages();
+  }
 }
 
 void MessagesView::contextMenuEvent(QContextMenuEvent* event) {
@@ -190,7 +265,7 @@ void MessagesView::contextMenuEvent(QContextMenuEvent* event) {
 
 void MessagesView::initializeContextMenu() {
   if (m_contextMenu == nullptr) {
-    m_contextMenu = new QMenu(tr("Context menu for messages"), this);
+    m_contextMenu = new QMenu(tr("Context menu for articles"), this);
   }
 
   m_contextMenu->clear();
@@ -209,10 +284,11 @@ void MessagesView::initializeContextMenu() {
   // External tools.
   QFileIconProvider icon_provider;
   QMenu* menu_ext_tools = new QMenu(tr("Open with external tool"), m_contextMenu);
+  auto tools = ExternalTool::toolsFromSettings();
 
   menu_ext_tools->setIcon(qApp->icons()->fromTheme(QSL("document-open")));
 
-  for (const ExternalTool& tool : ExternalTool::toolsFromSettings()) {
+  for (const ExternalTool& tool : qAsConst(tools)) {
     QAction* act_tool = new QAction(QFileInfo(tool.executable()).fileName(), menu_ext_tools);
 
     act_tool->setIcon(icon_provider.icon(tool.executable()));
@@ -224,7 +300,7 @@ void MessagesView::initializeContextMenu() {
   }
 
   if (menu_ext_tools->actions().isEmpty()) {
-    QAction* act_not_tools = new QAction("No external tools activated");
+    QAction* act_not_tools = new QAction(tr("No external tools activated"));
 
     act_not_tools->setEnabled(false);
     menu_ext_tools->addAction(act_not_tools);
@@ -275,7 +351,9 @@ void MessagesView::initializeContextMenu() {
 }
 
 void MessagesView::mousePressEvent(QMouseEvent* event) {
+  m_processingMouse = true;
   QTreeView::mousePressEvent(event);
+  m_processingMouse = false;
 
   switch (event->button()) {
     case Qt::MouseButton::LeftButton: {
@@ -332,7 +410,7 @@ void MessagesView::selectionChanged(const QItemSelection& selected, const QItemS
            << current_index << "', source '"
            << mapped_current_index << "'.";
 
-  if (mapped_current_index.isValid() && selected_rows.count() > 0) {
+  if (mapped_current_index.isValid() && selected_rows.size() == 1) {
     Message message = m_sourceModel->messageAt(m_proxyModel->mapToSource(current_index).row());
 
     // Set this message as read only if current item
@@ -346,8 +424,13 @@ void MessagesView::selectionChanged(const QItemSelection& selected, const QItemS
     emit currentMessageRemoved();
   }
 
-  if (qApp->settings()->value(GROUP(Messages), SETTING(Messages::KeepCursorInCenter)).toBool()) {
-    scrollTo(currentIndex(), QAbstractItemView::PositionAtCenter);
+  if (selected_rows.isEmpty()) {
+    setCurrentIndex({});
+  }
+
+  if (!m_processingMouse &&
+      qApp->settings()->value(GROUP(Messages), SETTING(Messages::KeepCursorInCenter)).toBool()) {
+    scrollTo(currentIndex(), QAbstractItemView::ScrollHint::PositionAtCenter);
   }
 
   QTreeView::selectionChanged(selected, deselected);
@@ -358,7 +441,7 @@ void MessagesView::loadItem(RootItem* item) {
   const Qt::SortOrder ord = header()->sortIndicatorOrder();
 
   scrollToTop();
-  sort(col, ord, false, true, false);
+  sort(col, ord, false, true, false, true);
   m_sourceModel->loadMessages(item);
 
   // Messages are loaded, make sure that previously
@@ -375,10 +458,12 @@ void MessagesView::switchShowUnreadOnly(bool set_new_value, bool show_unread_onl
 }
 
 void MessagesView::openSelectedSourceMessagesExternally() {
-  for (const QModelIndex& index : selectionModel()->selectedRows()) {
+  auto rws = selectionModel()->selectedRows();
+
+  for (const QModelIndex& index : qAsConst(rws)) {
     QString link = m_sourceModel->messageAt(m_proxyModel->mapToSource(index).row())
                    .m_url
-                   .replace(QRegularExpression("[\\t\\n]"), QString());
+                   .replace(QRegularExpression(QSL("[\\t\\n]")), QString());
 
     qApp->web()->openUrlInExternalBrowser(link);
   }
@@ -397,8 +482,9 @@ void MessagesView::openSelectedSourceMessagesExternally() {
 
 void MessagesView::openSelectedMessagesInternally() {
   QList<Message> messages;
+  auto rws = selectionModel()->selectedRows();
 
-  for (const QModelIndex& index : selectionModel()->selectedRows()) {
+  for (const QModelIndex& index : qAsConst(rws)) {
     messages << m_sourceModel->messageAt(m_proxyModel->mapToSource(index).row());
   }
 
@@ -427,19 +513,18 @@ void MessagesView::markSelectedMessagesUnread() {
 }
 
 void MessagesView::setSelectedMessagesReadStatus(RootItem::ReadStatus read) {
-  QModelIndex current_index = selectionModel()->currentIndex();
+  const QModelIndexList selected_indexes = selectionModel()->selectedRows();
 
-  if (!current_index.isValid()) {
+  if (selected_indexes.isEmpty()) {
     return;
   }
 
-  QModelIndexList selected_indexes = selectionModel()->selectedRows();
   const QModelIndexList mapped_indexes = m_proxyModel->mapListToSource(selected_indexes);
 
   m_sourceModel->setBatchMessagesRead(mapped_indexes, read);
-  current_index = m_proxyModel->index(current_index.row(), current_index.column());
+  QModelIndex current_index = selectionModel()->currentIndex();
 
-  if (current_index.isValid()) {
+  if (current_index.isValid() && selected_indexes.size() == 1) {
     emit currentMessageChanged(m_sourceModel->messageAt(m_proxyModel->mapToSource(current_index).row()), m_sourceModel->loadedItem());
   }
   else {
@@ -448,19 +533,20 @@ void MessagesView::setSelectedMessagesReadStatus(RootItem::ReadStatus read) {
 }
 
 void MessagesView::deleteSelectedMessages() {
-  QModelIndex current_index = selectionModel()->currentIndex();
+  const QModelIndexList selected_indexes = selectionModel()->selectedRows();
 
-  if (!current_index.isValid()) {
+  if (selected_indexes.isEmpty()) {
     return;
   }
 
-  const QModelIndexList selected_indexes = selectionModel()->selectedRows();
   const QModelIndexList mapped_indexes = m_proxyModel->mapListToSource(selected_indexes);
 
   m_sourceModel->setBatchMessagesDeleted(mapped_indexes);
-  current_index = moveCursor(QAbstractItemView::MoveDown, Qt::NoModifier);
+  QModelIndex current_index = currentIndex().isValid()
+                              ? moveCursor(QAbstractItemView::CursorAction::MoveDown, Qt::KeyboardModifier::NoModifier)
+                              : currentIndex();
 
-  if (current_index.isValid()) {
+  if (current_index.isValid() && selected_indexes.size() == 1) {
     setCurrentIndex(current_index);
   }
   else {
@@ -490,19 +576,18 @@ void MessagesView::restoreSelectedMessages() {
 }
 
 void MessagesView::switchSelectedMessagesImportance() {
-  QModelIndex current_index = selectionModel()->currentIndex();
+  const QModelIndexList selected_indexes = selectionModel()->selectedRows();
 
-  if (!current_index.isValid()) {
+  if (selected_indexes.isEmpty()) {
     return;
   }
 
-  QModelIndexList selected_indexes = selectionModel()->selectedRows();
   const QModelIndexList mapped_indexes = m_proxyModel->mapListToSource(selected_indexes);
 
   m_sourceModel->switchBatchMessageImportance(mapped_indexes);
-  current_index = m_proxyModel->index(current_index.row(), current_index.column());
+  QModelIndex current_index = selectionModel()->currentIndex();
 
-  if (current_index.isValid()) {
+  if (current_index.isValid() && selected_indexes.size() == 1) {
     emit currentMessageChanged(m_sourceModel->messageAt(m_proxyModel->mapToSource(current_index).row()), m_sourceModel->loadedItem());
   }
   else {
@@ -529,6 +614,12 @@ void MessagesView::selectNextItem() {
 
   if (index_next.isValid()) {
     setCurrentIndex(index_next);
+
+    scrollTo(index_next,
+             !m_processingMouse && qApp->settings()->value(GROUP(Messages), SETTING(Messages::KeepCursorInCenter)).toBool()
+             ? QAbstractItemView::ScrollHint::PositionAtCenter
+             : QAbstractItemView::ScrollHint::PositionAtTop);
+
     selectionModel()->select(index_next, QItemSelectionModel::Select | QItemSelectionModel::Rows);
     setFocus();
   }
@@ -539,6 +630,12 @@ void MessagesView::selectPreviousItem() {
 
   if (index_previous.isValid()) {
     setCurrentIndex(index_previous);
+
+    scrollTo(index_previous,
+             !m_processingMouse && qApp->settings()->value(GROUP(Messages), SETTING(Messages::KeepCursorInCenter)).toBool()
+             ? QAbstractItemView::ScrollHint::PositionAtCenter
+             : QAbstractItemView::ScrollHint::PositionAtTop);
+
     selectionModel()->select(index_previous, QItemSelectionModel::Select | QItemSelectionModel::Rows);
     setFocus();
   }
@@ -561,16 +658,32 @@ void MessagesView::selectNextUnreadItem() {
   if (next_unread.isValid()) {
     // We found unread message, mark it.
     setCurrentIndex(next_unread);
-    selectionModel()->select(next_unread, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
+
+    // Make sure that item is properly visible even if
+    // message previewer was hidden and shows up.
+    qApp->processEvents();
+
+    scrollTo(next_unread,
+             !m_processingMouse && qApp->settings()->value(GROUP(Messages), SETTING(Messages::KeepCursorInCenter)).toBool()
+             ? QAbstractItemView::ScrollHint::PositionAtCenter
+             : QAbstractItemView::ScrollHint::PositionAtTop);
+
+    selectionModel()->select(next_unread,
+                             QItemSelectionModel::SelectionFlag::Select |
+                             QItemSelectionModel::SelectionFlag::Rows);
     setFocus();
   }
 }
 
 void MessagesView::searchMessages(const QString& pattern) {
-#if QT_VERSION_MAJOR <= 5
-  m_proxyModel->setFilterRegExp(pattern);
+  qDebugNN << LOGSEC_GUI
+           << "Running search of messages with pattern"
+           << QUOTE_W_SPACE_DOT(pattern);
+
+#if QT_VERSION < 0x050C00 // Qt < 5.12.0
+  m_proxyModel->setFilterRegExp(pattern.toLower());
 #else
-  m_proxyModel->setFilterRegularExpression(pattern);
+  m_proxyModel->setFilterRegularExpression(pattern.toLower());
 #endif
 
   if (selectionModel()->selectedRows().isEmpty()) {
@@ -578,7 +691,10 @@ void MessagesView::searchMessages(const QString& pattern) {
   }
   else {
     // Scroll to selected message, it could become scrolled out due to filter change.
-    scrollTo(selectionModel()->selectedRows().at(0));
+    scrollTo(selectionModel()->selectedRows().at(0),
+             !m_processingMouse && qApp->settings()->value(GROUP(Messages), SETTING(Messages::KeepCursorInCenter)).toBool()
+             ? QAbstractItemView::ScrollHint::PositionAtCenter
+             : QAbstractItemView::ScrollHint::EnsureVisible);
   }
 }
 
@@ -591,17 +707,19 @@ void MessagesView::openSelectedMessagesWithExternalTool() {
 
   if (sndr != nullptr) {
     auto tool = sndr->data().value<ExternalTool>();
+    auto rws = selectionModel()->selectedRows();
 
-    for (const QModelIndex& index : selectionModel()->selectedRows()) {
+    for (const QModelIndex& index : qAsConst(rws)) {
       const QString link = m_sourceModel->messageAt(m_proxyModel->mapToSource(index).row())
                            .m_url
-                           .replace(QRegularExpression("[\\t\\n]"), QString());
+                           .replace(QRegularExpression(QSL("[\\t\\n]")), QString());
 
       if (!link.isEmpty()) {
-        if (!QProcess::startDetached(tool.executable(), QStringList() << tool.parameters() << link)) {
-          qApp->showGuiMessage(tr("Cannot run external tool"),
+        if (!tool.run(link)) {
+          qApp->showGuiMessage(Notification::Event::GeneralEvent,
+                               tr("Cannot run external tool"),
                                tr("External tool '%1' could not be started.").arg(tool.executable()),
-                               QSystemTrayIcon::Critical);
+                               QSystemTrayIcon::MessageIcon::Critical);
         }
       }
     }
@@ -614,13 +732,10 @@ void MessagesView::adjustColumns() {
 
     // Setup column resize strategies.
     for (int i = 0; i < header()->count(); i++) {
-      header()->setSectionResizeMode(i, QHeaderView::Interactive);
+      header()->setSectionResizeMode(i, QHeaderView::ResizeMode::Interactive);
     }
 
-    header()->setSectionResizeMode(MSG_DB_TITLE_INDEX, QHeaderView::Stretch);
-    header()->setSectionResizeMode(MSG_DB_READ_INDEX, QHeaderView::ResizeToContents);
-    header()->setSectionResizeMode(MSG_DB_IMPORTANT_INDEX, QHeaderView::ResizeToContents);
-    header()->setSectionResizeMode(MSG_DB_HAS_ENCLOSURES, QHeaderView::ResizeToContents);
+    header()->setSectionResizeMode(MSG_DB_TITLE_INDEX, QHeaderView::ResizeMode::Stretch);
 
     // Hide columns.
     hideColumn(MSG_DB_ID_INDEX);
@@ -629,16 +744,19 @@ void MessagesView::adjustColumns() {
     hideColumn(MSG_DB_CONTENTS_INDEX);
     hideColumn(MSG_DB_PDELETED_INDEX);
     hideColumn(MSG_DB_ENCLOSURES_INDEX);
+    hideColumn(MSG_DB_SCORE_INDEX);
     hideColumn(MSG_DB_ACCOUNT_ID_INDEX);
     hideColumn(MSG_DB_CUSTOM_ID_INDEX);
     hideColumn(MSG_DB_CUSTOM_HASH_INDEX);
     hideColumn(MSG_DB_FEED_CUSTOM_ID_INDEX);
+    hideColumn(MSG_DB_FEED_TITLE_INDEX);
+    hideColumn(MSG_DB_HAS_ENCLOSURES);
   }
 }
 
 void MessagesView::onSortIndicatorChanged(int column, Qt::SortOrder order) {
   // Repopulate the shit.
-  sort(column, order, true, false, false);
+  sort(column, order, true, false, false, false);
 
   emit currentMessageRemoved();
 }
