@@ -7,12 +7,15 @@
 #include "gui/tabwidget.h"
 #include "gui/webbrowser.h"
 #include "miscellaneous/application.h"
+#include "miscellaneous/externaltool.h"
 #include "miscellaneous/skinfactory.h"
 #include "network-web/adblock/adblockicon.h"
 #include "network-web/adblock/adblockmanager.h"
+#include "network-web/networkfactory.h"
 #include "network-web/webfactory.h"
 #include "network-web/webpage.h"
 
+#include <QFileIconProvider>
 #include <QOpenGLWidget>
 #include <QTimer>
 #include <QWebEngineContextMenuData>
@@ -51,7 +54,7 @@ WebPage* WebViewer::page() const {
 }
 
 void WebViewer::displayMessage() {
-  setHtml(m_messageContents, QUrl::fromUserInput(INTERNAL_URL_MESSAGE));
+  setHtml(m_messageContents, m_messageBaseUrl /*, QUrl::fromUserInput(INTERNAL_URL_MESSAGE)*/);
 }
 
 bool WebViewer::increaseWebPageZoom() {
@@ -76,9 +79,13 @@ bool WebViewer::decreaseWebPageZoom() {
   }
 }
 
-bool WebViewer::resetWebPageZoom() {
-  const qreal new_factor = qApp->settings()->value(GROUP(Messages),
-                                                   SETTING(Messages::Zoom)).toReal();
+bool WebViewer::resetWebPageZoom(bool to_factory_default) {
+  const qreal new_factor = to_factory_default ? 1.0 : qApp->settings()->value(GROUP(Messages),
+                                                                              SETTING(Messages::Zoom)).toReal();
+
+  if (to_factory_default) {
+    qApp->settings()->setValue(GROUP(Messages), Messages::Zoom, new_factor);
+  }
 
   if (new_factor != zoomFactor()) {
     setZoomFactor(new_factor);
@@ -102,7 +109,7 @@ void WebViewer::loadMessages(const QList<Message>& messages, RootItem* root) {
       QString enc_url;
 
       if (!enclosure.m_url.contains(QRegularExpression(QSL("^(http|ftp|\\/)")))) {
-        enc_url = QString(INTERNAL_URL_PASSATTACHMENT) + QL1S("/?") + enclosure.m_url;
+        enc_url = QSL(INTERNAL_URL_PASSATTACHMENT) + QL1S("/?") + enclosure.m_url;
       }
       else {
         enc_url = enclosure.m_url;
@@ -111,9 +118,11 @@ void WebViewer::loadMessages(const QList<Message>& messages, RootItem* root) {
       enc_url = QUrl::fromPercentEncoding(enc_url.toUtf8());
 
       enclosures += skin.m_enclosureMarkup.arg(enc_url,
-                                               QSL("&#129527;"), enclosure.m_mimeType);
+                                               QSL("&#129527;"),
+                                               enclosure.m_mimeType);
 
-      if (enclosure.m_mimeType.startsWith(QSL("image/"))) {
+      if (enclosure.m_mimeType.startsWith(QSL("image/")) &&
+          qApp->settings()->value(GROUP(Messages), SETTING(Messages::DisplayEnclosuresInMessage)).toBool()) {
         // Add thumbnail image.
         enclosure_images += skin.m_enclosureImageMarkup.arg(
           enclosure.m_url,
@@ -122,6 +131,11 @@ void WebViewer::loadMessages(const QList<Message>& messages, RootItem* root) {
       }
     }
 
+    QString msg_date = qApp->settings()->value(GROUP(Messages), SETTING(Messages::UseCustomDate)).toBool()
+                       ? message.m_created.toLocalTime().toString(qApp->settings()->value(GROUP(Messages),
+                                                                                          SETTING(Messages::CustomDateFormat)).toString())
+                       : QLocale().toString(message.m_created.toLocalTime(), QLocale::FormatType::ShortFormat);
+
     messages_layout.append(single_message_layout
                            .arg(message.m_title,
                                 tr("Written by ") + (message.m_author.isEmpty() ?
@@ -129,12 +143,27 @@ void WebViewer::loadMessages(const QList<Message>& messages, RootItem* root) {
                                                      message.m_author),
                                 message.m_url,
                                 message.m_contents,
-                                QLocale().toString(message.m_created, QLocale::FormatType::ShortFormat),
+                                msg_date,
                                 enclosures,
                                 enclosure_images));
   }
 
   m_root = root;
+
+  auto* feed = root->getParentServiceRoot()->getItemFromSubTree([messages](const RootItem* it) {
+    return it->kind() == RootItem::Kind::Feed && it->customId() == messages.at(0).m_feedId;
+  })->toFeed();
+
+  m_messageBaseUrl = QString();
+
+  if (feed != nullptr) {
+    QUrl url(NetworkFactory::sanitizeUrl(feed->source()));
+
+    if (url.isValid()) {
+      m_messageBaseUrl = url.scheme() + QSL("://") + url.host();
+    }
+  }
+
   m_messageContents = skin.m_layoutMarkupWrapper.arg(messages.size() == 1 ? messages.at(0).m_title : tr("Newspaper view"),
                                                      messages_layout);
 
@@ -151,7 +180,7 @@ void WebViewer::clear() {
   bool previously_enabled = isEnabled();
 
   setEnabled(false);
-  setHtml("<!DOCTYPE html><html><body</body></html>", QUrl(INTERNAL_URL_BLANK));
+  setHtml(QSL("<!DOCTYPE html><html><body</body></html>"), QUrl(QSL(INTERNAL_URL_BLANK)));
   setEnabled(previously_enabled);
 }
 
@@ -162,7 +191,7 @@ void WebViewer::contextMenuEvent(QContextMenuEvent* event) {
 
   if (menu_data.linkUrl().isValid()) {
     // Add option to open link in external viewe
-    menu->addAction(qApp->icons()->fromTheme(QSL("")), tr("Open link in external browser"), [menu_data]() {
+    menu->addAction(qApp->icons()->fromTheme(QSL("document-open")), tr("Open link in external browser"), [menu_data]() {
       qApp->web()->openUrlInExternalBrowser(menu_data.linkUrl().toString());
 
       if (qApp->settings()->value(GROUP(Messages), SETTING(Messages::BringAppToFrontAfterMessageOpenedExternally)).toBool()) {
@@ -173,6 +202,36 @@ void WebViewer::contextMenuEvent(QContextMenuEvent* event) {
     });
   }
 
+  if (menu_data.mediaUrl().isValid() || menu_data.linkUrl().isValid()) {
+    QFileIconProvider icon_provider;
+    QMenu* menu_ext_tools = new QMenu(tr("Open with external tool"), menu);
+    auto tools = ExternalTool::toolsFromSettings();
+
+    menu_ext_tools->setIcon(qApp->icons()->fromTheme(QSL("document-open")));
+
+    for (const ExternalTool& tool : qAsConst(tools)) {
+      QAction* act_tool = new QAction(QFileInfo(tool.executable()).fileName(), menu_ext_tools);
+
+      act_tool->setIcon(icon_provider.icon(tool.executable()));
+      act_tool->setToolTip(tool.executable());
+      act_tool->setData(QVariant::fromValue(tool));
+      menu_ext_tools->addAction(act_tool);
+
+      connect(act_tool, &QAction::triggered, this, [this, act_tool, menu_data]() {
+        openUrlWithExternalTool(act_tool->data().value<ExternalTool>(), menu_data);
+      });
+    }
+
+    if (menu_ext_tools->actions().isEmpty()) {
+      QAction* act_not_tools = new QAction(tr("No external tools activated"));
+
+      act_not_tools->setEnabled(false);
+      menu_ext_tools->addAction(act_not_tools);
+    }
+
+    menu->addMenu(menu_ext_tools);
+  }
+
   menu->addAction(qApp->web()->adBlock()->adBlockIcon());
   menu->addAction(qApp->web()->engineSettingsAction());
 
@@ -227,12 +286,20 @@ bool WebViewer::eventFilter(QObject* object, QEvent* event) {
         decreaseWebPageZoom();
         return true;
       }
+      else if (key_event->key() == Qt::Key::Key_0) {
+        resetWebPageZoom(true);
+        return true;
+      }
     }
   }
 
   return false;
 }
 
+void WebViewer::openUrlWithExternalTool(ExternalTool tool, const QWebEngineContextMenuData& target) {
+  tool.run(target.mediaUrl().isValid() ? target.mediaUrl().toString() : target.linkUrl().toString());
+}
+
 RootItem* WebViewer::root() const {
   return m_root;
 }
