@@ -2,11 +2,11 @@
 
 #include "miscellaneous/textfactory.h"
 
+#include "3rd-party/sc/simplecrypt.h"
 #include "definitions/definitions.h"
 #include "exceptions/applicationexception.h"
 #include "miscellaneous/application.h"
 #include "miscellaneous/iofactory.h"
-#include "miscellaneous/simplecrypt/simplecrypt.h"
 
 #include <QDir>
 #include <QLocale>
@@ -71,7 +71,7 @@ QDateTime TextFactory::parseDateTime(const QString& date_time) {
   const QString input_date = date_time.simplified();
   QDateTime dt;
   QTime time_zone_offset;
-  const QLocale locale(QLocale::C);
+  const QLocale locale(QLocale::Language::C);
   bool positive_time_zone_offset = false;
   QStringList date_patterns;
 
@@ -79,25 +79,25 @@ QDateTime TextFactory::parseDateTime(const QString& date_time) {
     QSL("MMM d yyyy hh:mm:ss") << QSL("ddd, dd MMM yyyy HH:mm:ss") << QSL("ddd, d MMM yyyy HH:mm:ss") <<
     QSL("dd MMM yyyy") << QSL("yyyy-MM-dd HH:mm:ss.z") << QSL("yyyy-MM-dd") <<
     QSL("yyyy") << QSL("yyyy-MM") << QSL("yyyy-MM-dd") << QSL("yyyy-MM-ddThh:mm") <<
-    QSL("yyyy-MM-ddThh:mm:ss");
+    QSL("yyyy-MM-ddThh:mm:ss") << QSL("d MMM yyyy HH:mm:ss");
   QStringList timezone_offset_patterns;
 
   timezone_offset_patterns << QSL("+hh:mm") << QSL("-hh:mm") << QSL("+hhmm")
                            << QSL("-hhmm") << QSL("+hh") << QSL("-hh");
 
   // Iterate over patterns and check if input date/time matches the pattern.
-  for (const QString& pattern : date_patterns) {
+  for (const QString& pattern : qAsConst(date_patterns)) {
     dt = locale.toDateTime(input_date.left(pattern.size()), pattern);
 
     if (dt.isValid()) {
       // Make sure that this date/time is considered UTC.
-      dt.setTimeSpec(Qt::UTC);
+      dt.setTimeSpec(Qt::TimeSpec::UTC);
 
       // We find offset from UTC.
       if (input_date.size() >= TIMEZONE_OFFSET_LIMIT) {
         QString offset_sanitized = input_date.mid(pattern.size()).replace(QL1S(" "), QString());
 
-        for (const QString& pattern_t : timezone_offset_patterns) {
+        for (const QString& pattern_t : qAsConst(timezone_offset_patterns)) {
           time_zone_offset = QTime::fromString(offset_sanitized.left(pattern_t.size()), pattern_t);
 
           if (time_zone_offset.isValid()) {
@@ -133,12 +133,12 @@ QDateTime TextFactory::parseDateTime(qint64 milis_from_epoch) {
   return QDateTime::fromMSecsSinceEpoch(milis_from_epoch, Qt::TimeSpec::UTC);
 }
 
-QString TextFactory::encrypt(const QString& text) {
-  return SimpleCrypt(initializeSecretEncryptionKey()).encryptToString(text);
+QString TextFactory::encrypt(const QString& text, quint64 key) {
+  return SimpleCrypt(key == 0 ? initializeSecretEncryptionKey() : key).encryptToString(text);
 }
 
-QString TextFactory::decrypt(const QString& text) {
-  return SimpleCrypt(initializeSecretEncryptionKey()).decryptToString(text);
+QString TextFactory::decrypt(const QString& text, quint64 key) {
+  return SimpleCrypt(key == 0 ? initializeSecretEncryptionKey() : key).decryptToString(text);
 }
 
 QString TextFactory::newline() {
