@@ -26,27 +26,27 @@ void BaseNetworkAccessManager::loadSettings() {
     setProxy(QNetworkProxy::ProxyType::NoProxy);
   }
   else {
+    qWarningNN << LOGSEC_NETWORK << "Using application-wide proxy.";
+
+    if (QNetworkProxy::applicationProxy().type() != QNetworkProxy::ProxyType::DefaultProxy &&
+        QNetworkProxy::applicationProxy().type() != QNetworkProxy::ProxyType::NoProxy) {
+      qWarningNN << LOGSEC_NETWORK
+                 << "Used proxy address:"
+                 << QUOTE_W_SPACE_COMMA(QNetworkProxy::applicationProxy().hostName())
+                 << " type:"
+                 << QUOTE_W_SPACE_DOT(QNetworkProxy::applicationProxy().type());
+    }
+
     setProxy(QNetworkProxy::applicationProxy());
   }
 
   qDebugNN << LOGSEC_NETWORK << "Settings of BaseNetworkAccessManager loaded.";
 }
 
-void BaseNetworkAccessManager::acceptRedirection(const QUrl& url) {
-  auto* reply = qobject_cast<QNetworkReply*>(sender());
-  emit reply->redirectAllowed();
-
-  qDebugNN << LOGSEC_NETWORK << "Accepting redirect to" << QUOTE_W_SPACE_DOT(url.toString());
-}
-
 void BaseNetworkAccessManager::onSslErrors(QNetworkReply* reply, const QList<QSslError>& error) {
   qWarningNN << LOGSEC_NETWORK
              << "Ignoring SSL errors for"
-             << QUOTE_W_SPACE(reply->url().toString())
-             << ":"
-             << QUOTE_W_SPACE(reply->errorString())
-             << "- code"
-             << QUOTE_W_SPACE_DOT(reply->error());
+             << QUOTE_W_SPACE_DOT(reply->url().toString());
   reply->ignoreSslErrors(error);
 }
 
@@ -60,17 +60,9 @@ QNetworkReply* BaseNetworkAccessManager::createRequest(QNetworkAccessManager::Op
   new_request.setAttribute(QNetworkRequest::Attribute::Http2AllowedAttribute, true);
 #endif
 
-#if QT_VERSION >= 0x050900
-  new_request.setAttribute(QNetworkRequest::Attribute::RedirectPolicyAttribute,
-                           QNetworkRequest::RedirectPolicy::UserVerifiedRedirectPolicy);
-#elif QT_VERSION >= 0x050600
-  new_request.setAttribute(QNetworkRequest::Attribute::FollowRedirectsAttribute, true);
-#endif
-
   new_request.setRawHeader(HTTP_HEADERS_COOKIE, QSL("JSESSIONID= ").toLocal8Bit());
-  new_request.setRawHeader(HTTP_HEADERS_USER_AGENT, QString(APP_USERAGENT).toLocal8Bit());
+  new_request.setRawHeader(HTTP_HEADERS_USER_AGENT, QSL(APP_USERAGENT).toLocal8Bit());
 
   auto reply = QNetworkAccessManager::createRequest(op, new_request, outgoingData);
-  connect(reply, &QNetworkReply::redirected, this, &BaseNetworkAccessManager::acceptRedirection);
   return reply;
 }
