@@ -141,7 +141,7 @@ QString DownloadItem::saveFileName(const QString& directory) const {
 
   if (m_reply->hasRawHeader("Content-Disposition")) {
     QString value = QLatin1String(m_reply->rawHeader("Content-Disposition"));
-    QRegularExpression exp(".*filename=?\"([^\"]+)\"?");
+    QRegularExpression exp(QSL(".*filename\\s*=\\s*\"?([^\"]+)\"?"));
     QRegularExpressionMatch match = exp.match(value);
 
     if (match.isValid()) {
@@ -194,8 +194,11 @@ void DownloadItem::stop() {
 
 void DownloadItem::openFile() {
   if (!QDesktopServices::openUrl(QUrl::fromLocalFile(m_output.fileName()))) {
-    qApp->showGuiMessage(tr("Cannot open file"), tr("Cannot open output file. Open it manually."),
-                         QSystemTrayIcon::Warning, qApp->mainFormWidget(), true);
+    qApp->showGuiMessage(Notification::Event::GeneralEvent,
+                         tr("Cannot open file"),
+                         tr("Cannot open output file. Open it manually."),
+                         QSystemTrayIcon::MessageIcon::Warning,
+                         true);
   }
 }
 
@@ -281,7 +284,7 @@ void DownloadItem::error(QNetworkReply::NetworkError code) {
 }
 
 void DownloadItem::metaDataChanged() {
-  QVariant locationHeader = m_reply->header(QNetworkRequest::LocationHeader);
+  QVariant locationHeader = m_reply->header(QNetworkRequest::KnownHeaders::LocationHeader);
 
   if (locationHeader.isValid()) {
     m_url = locationHeader.toUrl();
@@ -317,7 +320,7 @@ void DownloadItem::downloadProgress(qint64 bytes_received, qint64 bytes_total) {
 }
 
 qint64 DownloadItem::bytesTotal() const {
-  return m_reply->header(QNetworkRequest::ContentLengthHeader).toULongLong();
+  return m_reply == nullptr ? 0 : m_reply->header(QNetworkRequest::ContentLengthHeader).toULongLong();
 }
 
 qint64 DownloadItem::bytesReceived() const {
@@ -385,7 +388,9 @@ void DownloadItem::updateDownloadInfoLabel() {
 }
 
 bool DownloadItem::downloading() const {
-  return (m_ui->m_progressDownload->isVisible());
+  return !m_finishedDownloading;
+
+  //return (m_ui->m_progressDownload->isVisible());
 }
 
 bool DownloadItem::downloadedSuccessfully() const {
@@ -410,12 +415,14 @@ void DownloadItem::finished() {
   emit downloadFinished();
 
   if (downloadedSuccessfully()) {
-    qApp->showGuiMessage(tr("Download finished"),
+    qApp->showGuiMessage(Notification::Event::GeneralEvent,
+                         tr("Download finished"),
                          tr("File '%1' is downloaded.\nClick here to open parent directory.").arg(QDir::toNativeSeparators(
                                                                                                     m_output.fileName())),
                          QSystemTrayIcon::MessageIcon::Information,
-                         nullptr,
-                         false,
+                         {},
+                         {},
+                         tr("Open folder"),
                          [this] {
       openFolder();
     });
@@ -497,7 +504,7 @@ void DownloadManager::handleUnsupportedContent(QNetworkReply* reply) {
     return;
   }
 
-  const QVariant header = reply->header(QNetworkRequest::ContentLengthHeader);
+  const QVariant header = reply->header(QNetworkRequest::KnownHeaders::ContentLengthHeader);
   bool ok;
   const int size = header.toInt(&ok);
 
@@ -526,7 +533,7 @@ void DownloadManager::addItem(DownloadItem* item) {
   m_downloads.append(item);
   m_model->endInsertRows();
   m_ui->m_viewDownloads->setIndexWidget(m_model->index(row, 0), item);
-  QIcon icon = style()->standardIcon(QStyle::SP_FileIcon);
+  QIcon icon = style()->standardIcon(QStyle::StandardPixmap::SP_FileIcon);
 
   item->m_ui->m_lblFileIcon->setPixmap(icon.pixmap(DOWNLOADER_ICON_SIZE, DOWNLOADER_ICON_SIZE));
   m_ui->m_viewDownloads->setRowHeight(row, item->sizeHint().height());
@@ -578,7 +585,7 @@ void DownloadManager::updateRow(DownloadItem* item) {
   QIcon icon = m_iconProvider->icon(item->m_output.fileName());
 
   if (icon.isNull()) {
-    icon = style()->standardIcon(QStyle::SP_FileIcon);
+    icon = style()->standardIcon(QStyle::StandardPixmap::SP_FileIcon);
   }
 
   item->m_ui->m_lblFileIcon->setPixmap(icon.pixmap(DOWNLOADER_ICON_SIZE, DOWNLOADER_ICON_SIZE));
@@ -730,7 +737,7 @@ QString DownloadManager::dataString(qint64 size) {
     unit = tr("GB");
   }
 
-  return QString(QL1S("%1 %2")).arg(new_size, 0, 'f', 1).arg(unit);
+  return QSL("%1 %2").arg(new_size, 0, 'f', 1).arg(unit);
 }
 
 DownloadModel::DownloadModel(DownloadManager* download_manager, QObject* parent)
