@@ -28,6 +28,7 @@ class NetworkFactory {
 
     // Returns human readable text for given network error.
     static QString networkErrorText(QNetworkReply::NetworkError error_code);
+    static QString sanitizeUrl(const QString& url);
 
     // Performs SYNCHRONOUS download if favicon for the site,
     // given URL belongs to.
@@ -39,8 +40,7 @@ class NetworkFactory {
                                                  const QByteArray& input_data,
                                                  QByteArray& output,
                                                  QNetworkAccessManager::Operation operation,
-                                                 QList<QPair<QByteArray,
-                                                             QByteArray>> additional_headers = QList<QPair<QByteArray, QByteArray>>(),
+                                                 const QList<QPair<QByteArray, QByteArray>>& additional_headers = QList<QPair<QByteArray, QByteArray>>(),
                                                  bool protected_contents = false,
                                                  const QString& username = QString(),
                                                  const QString& password = QString(),
@@ -49,8 +49,7 @@ class NetworkFactory {
                                                  QHttpMultiPart* input_data,
                                                  QList<HttpResponse>& output,
                                                  QNetworkAccessManager::Operation operation,
-                                                 QList<QPair<QByteArray,
-                                                             QByteArray>> additional_headers = QList<QPair<QByteArray, QByteArray>>(),
+                                                 const QList<QPair<QByteArray, QByteArray>>& additional_headers = QList<QPair<QByteArray, QByteArray>>(),
                                                  bool protected_contents = false,
                                                  const QString& username = QString(),
                                                  const QString& password = QString(),
