@@ -5,8 +5,9 @@
 #include "3rd-party/boolinq/boolinq.h"
 #include "core/feedsmodel.h"
 #include "core/messagesmodel.h"
+#include "database/databasequeries.h"
+#include "exceptions/applicationexception.h"
 #include "miscellaneous/application.h"
-#include "miscellaneous/databasequeries.h"
 #include "miscellaneous/iconfactory.h"
 #include "miscellaneous/textfactory.h"
 #include "services/abstract/cacheforserviceroot.h"
@@ -15,18 +16,22 @@
 #include "services/abstract/importantnode.h"
 #include "services/abstract/labelsnode.h"
 #include "services/abstract/recyclebin.h"
+#include "services/abstract/unreadnode.h"
+
+#include <QThread>
 
 ServiceRoot::ServiceRoot(RootItem* parent)
   : RootItem(parent), m_recycleBin(new RecycleBin(this)), m_importantNode(new ImportantNode(this)),
-  m_labelsNode(new LabelsNode(this)), m_accountId(NO_PARENT_CATEGORY) {
+  m_labelsNode(new LabelsNode(this)), m_unreadNode(new UnreadNode(this)),
+  m_accountId(NO_PARENT_CATEGORY), m_networkProxy(QNetworkProxy()) {
   setKind(RootItem::Kind::ServiceRoot);
-  setCreationDate(QDateTime::currentDateTime());
+  appendCommonNodes();
 }
 
 ServiceRoot::~ServiceRoot() = default;
 
 bool ServiceRoot::deleteViaGui() {
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
   if (DatabaseQueries::deleteAccount(database, accountId())) {
     stop();
@@ -45,7 +50,7 @@ bool ServiceRoot::markAsReadUnread(RootItem::ReadStatus status) {
     cache->addMessageStatesToCache(customIDSOfMessagesForItem(this), status);
   }
 
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
   if (DatabaseQueries::markAccountReadUnread(database, accountId(), status)) {
     updateCounts(false);
@@ -91,7 +96,7 @@ QList<QAction*> ServiceRoot::serviceMenu() {
       auto* cache = toCache();
 
       if (cache != nullptr) {
-        auto* act_sync_cache = new QAction(qApp->icons()->fromTheme(QSL("view-refresh")), tr("Synchronize message cache"), this);
+        auto* act_sync_cache = new QAction(qApp->icons()->fromTheme(QSL("view-refresh")), tr("Synchronize article cache"), this);
 
         connect(act_sync_cache, &QAction::triggered, this, [cache]() {
           cache->saveAllCachedData(false);
@@ -117,8 +122,9 @@ void ServiceRoot::stop() {}
 
 void ServiceRoot::updateCounts(bool including_total_count) {
   QList<Feed*> feeds;
+  auto str = getSubTree();
 
-  for (RootItem* child : getSubTree()) {
+  for (RootItem* child : qAsConst(str)) {
     if (child->kind() == RootItem::Kind::Feed) {
       feeds.append(child->toFeed());
     }
@@ -133,7 +139,7 @@ void ServiceRoot::updateCounts(bool including_total_count) {
     return;
   }
 
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
   bool ok;
   QMap<QString, QPair<int, int>> counts = DatabaseQueries::getMessageCountsForAccount(database, accountId(), including_total_count, &ok);
 
@@ -157,12 +163,16 @@ void ServiceRoot::updateCounts(bool including_total_count) {
   }
 }
 
+bool ServiceRoot::canBeDeleted() const {
+  return true;
+}
+
 void ServiceRoot::completelyRemoveAllData() {
   // Purge old data from SQL and clean all model items.
-  cleanAllItemsFromModel();
-  removeOldAccountFromDatabase(true);
+  cleanAllItemsFromModel(true);
+  removeOldAccountFromDatabase(true, true);
   updateCounts(true);
-  itemChanged(QList<RootItem*>() << this);
+  itemChanged({ this });
   requestReloadMessageList(true);
 }
 
@@ -180,30 +190,56 @@ QIcon ServiceRoot::feedIconForMessage(const QString& feed_custom_id) const {
   }
 }
 
-void ServiceRoot::removeOldAccountFromDatabase(bool including_messages) {
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+void ServiceRoot::removeOldAccountFromDatabase(bool delete_messages_too, bool delete_labels_too) {
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
-  DatabaseQueries::deleteAccountData(database, accountId(), including_messages);
+  DatabaseQueries::deleteAccountData(database,
+                                     accountId(),
+                                     delete_messages_too,
+                                     delete_labels_too);
 }
 
-void ServiceRoot::cleanAllItemsFromModel() {
-  for (RootItem* top_level_item : childItems()) {
+void ServiceRoot::cleanAllItemsFromModel(bool clean_labels_too) {
+  auto chi = childItems();
+
+  for (RootItem* top_level_item : qAsConst(chi)) {
     if (top_level_item->kind() != RootItem::Kind::Bin &&
         top_level_item->kind() != RootItem::Kind::Important &&
+        top_level_item->kind() != RootItem::Kind::Unread &&
         top_level_item->kind() != RootItem::Kind::Labels) {
       requestItemRemoval(top_level_item);
     }
   }
 
-  if (labelsNode() != nullptr) {
-    for (RootItem* lbl : labelsNode()->childItems()) {
+  if (labelsNode() != nullptr && clean_labels_too) {
+    auto lbl_chi = labelsNode()->childItems();
+
+    for (RootItem* lbl : qAsConst(lbl_chi)) {
       requestItemRemoval(lbl);
     }
   }
 }
 
-bool ServiceRoot::cleanFeeds(QList<Feed*> items, bool clean_read_only) {
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+void ServiceRoot::appendCommonNodes() {
+  if (recycleBin() != nullptr && !childItems().contains(recycleBin())) {
+    appendChild(recycleBin());
+  }
+
+  if (importantNode() != nullptr && !childItems().contains(importantNode())) {
+    appendChild(importantNode());
+  }
+
+  if (unreadNode() != nullptr && !childItems().contains(unreadNode())) {
+    appendChild(unreadNode());
+  }
+
+  if (labelsNode() != nullptr && !childItems().contains(labelsNode())) {
+    appendChild(labelsNode());
+  }
+}
+
+bool ServiceRoot::cleanFeeds(const QList<Feed*>& items, bool clean_read_only) {
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
   if (DatabaseQueries::cleanFeeds(database, textualFeedIds(items), clean_read_only, accountId())) {
     getParentServiceRoot()->updateCounts(true);
@@ -217,29 +253,34 @@ bool ServiceRoot::cleanFeeds(QList<Feed*> items, bool clean_read_only) {
 }
 
 void ServiceRoot::storeNewFeedTree(RootItem* root) {
-  DatabaseQueries::storeAccountTree(qApp->database()->connection(metaObject()->className()), root, accountId());
+  try {
+    DatabaseQueries::storeAccountTree(qApp->database()->driver()->connection(metaObject()->className()), root, accountId());
+  }
+  catch (const ApplicationException& ex) {
+    qFatal("Cannot store account tree: '%s'.", qPrintable(ex.message()));
+  }
 }
 
 void ServiceRoot::removeLeftOverMessages() {
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
   DatabaseQueries::purgeLeftoverMessages(database, accountId());
 }
 
 void ServiceRoot::removeLeftOverMessageFilterAssignments() {
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
   DatabaseQueries::purgeLeftoverMessageFilterAssignments(database, accountId());
 }
 
 void ServiceRoot::removeLeftOverMessageLabelAssignments() {
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
   DatabaseQueries::purgeLeftoverLabelAssignments(database, accountId());
 }
 
 QList<Message> ServiceRoot::undeletedMessages() const {
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
   return DatabaseQueries::getUndeletedMessagesForAccount(database, accountId());
 }
@@ -256,6 +297,37 @@ ServiceRoot::LabelOperation ServiceRoot::supportedLabelOperations() const {
   return LabelOperation::Adding | LabelOperation::Editing | LabelOperation::Deleting;
 }
 
+void ServiceRoot::saveAccountDataToDatabase() {
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
+
+  try {
+    DatabaseQueries::createOverwriteAccount(database, this);
+  }
+  catch (const ApplicationException& ex) {
+    qFatal("Account was not saved into database: '%s'.", qPrintable(ex.message()));
+  }
+}
+
+QVariantHash ServiceRoot::customDatabaseData() const {
+  return {};
+}
+
+void ServiceRoot::setCustomDatabaseData(const QVariantHash& data) {
+  Q_UNUSED(data)
+}
+
+bool ServiceRoot::wantsBaggedIdsOfExistingMessages() const {
+  return false;
+}
+
+void ServiceRoot::aboutToBeginFeedFetching(const QList<Feed*>& feeds,
+                                           const QHash<QString, QHash<BagOfMessages, QStringList>>& stated_messages,
+                                           const QHash<QString, QStringList>& tagged_messages) {
+  Q_UNUSED(feeds)
+  Q_UNUSED(stated_messages)
+  Q_UNUSED(tagged_messages)
+}
+
 void ServiceRoot::itemChanged(const QList<RootItem*>& items) {
   emit dataChanged(items);
 }
@@ -291,10 +363,13 @@ void ServiceRoot::addNewCategory(RootItem* selected_item) {
 
 QMap<QString, QVariantMap> ServiceRoot::storeCustomFeedsData() {
   QMap<QString, QVariantMap> custom_data;
+  auto str = getSubTreeFeeds();
 
-  for (const Feed* feed : getSubTreeFeeds()) {
+  for (const Feed* feed : qAsConst(str)) {
     QVariantMap feed_custom_data;
 
+    // TODO: This could potentially call Feed::customDatabaseData() and append it
+    // to this map and also subsequently restore.
     feed_custom_data.insert(QSL("auto_update_interval"), feed->autoUpdateInitialInterval());
     feed_custom_data.insert(QSL("auto_update_type"), int(feed->autoUpdateType()));
     feed_custom_data.insert(QSL("msg_filters"), QVariant::fromValue(feed->messageFilters()));
@@ -340,6 +415,10 @@ LabelsNode* ServiceRoot::labelsNode() const {
   return m_labelsNode;
 }
 
+UnreadNode* ServiceRoot::unreadNode() const {
+  return m_unreadNode;
+}
+
 void ServiceRoot::syncIn() {
   QIcon original_icon = icon();
 
@@ -351,8 +430,13 @@ void ServiceRoot::syncIn() {
     auto feed_custom_data = storeCustomFeedsData();
 
     // Remove from feeds model, then from SQL but leave messages intact.
-    cleanAllItemsFromModel();
-    removeOldAccountFromDatabase(false);
+    bool uses_remote_labels = (supportedLabelOperations() & LabelOperation::Synchronised) == LabelOperation::Synchronised;
+
+    // Remove stuff.
+    cleanAllItemsFromModel(uses_remote_labels);
+    removeOldAccountFromDatabase(false, uses_remote_labels);
+
+    // Restore some local settings to feeds etc.
     restoreCustomFeedsData(feed_custom_data, new_tree->getHashedSubTreeFeeds());
 
     // Model is clean, now store new tree into DB and
@@ -365,7 +449,9 @@ void ServiceRoot::syncIn() {
     removeLeftOverMessageFilterAssignments();
     removeLeftOverMessageLabelAssignments();
 
-    for (RootItem* top_level_item : new_tree->childItems()) {
+    auto chi = new_tree->childItems();
+
+    for (RootItem* top_level_item : qAsConst(chi)) {
       if (top_level_item->kind() != Kind::Labels) {
         top_level_item->setParent(nullptr);
         requestItemReassignment(top_level_item, this);
@@ -373,7 +459,9 @@ void ServiceRoot::syncIn() {
       else {
         // It seems that some labels got synced-in.
         if (labelsNode() != nullptr) {
-          for (RootItem* new_lbl : top_level_item->childItems()) {
+          auto lbl_chi = top_level_item->childItems();
+
+          for (RootItem* new_lbl : qAsConst(lbl_chi)) {
             new_lbl->setParent(nullptr);
             requestItemReassignment(new_lbl, labelsNode());
           }
@@ -390,21 +478,15 @@ void ServiceRoot::syncIn() {
 
   setIcon(original_icon);
   itemChanged(getSubTree());
+  requestItemExpand(getSubTree(), true);
 }
 
-void ServiceRoot::performInitialAssembly(const Assignment& categories, const Assignment& feeds, const QList<Label*>& labels) {
-  // All data are now obtained, lets create the hierarchy.
+void ServiceRoot::performInitialAssembly(const Assignment& categories,
+                                         const Assignment& feeds,
+                                         const QList<Label*>& labels) {
   assembleCategories(categories);
   assembleFeeds(feeds);
-
-  // As the last item, add recycle bin, which is needed.
-  appendChild(recycleBin());
-  appendChild(importantNode());
-  appendChild(labelsNode());
-
   labelsNode()->loadLabels(labels);
-  requestItemExpand({ labelsNode() }, true);
-
   updateCounts(true);
 }
 
@@ -423,7 +505,9 @@ QStringList ServiceRoot::customIDSOfMessagesForItem(RootItem* item) {
     switch (item->kind()) {
       case RootItem::Kind::Labels:
       case RootItem::Kind::Category: {
-        for (RootItem* child : item->childItems()) {
+        auto chi = item->childItems();
+
+        for (RootItem* child : qAsConst(chi)) {
           list.append(customIDSOfMessagesForItem(child));
         }
 
@@ -431,40 +515,47 @@ QStringList ServiceRoot::customIDSOfMessagesForItem(RootItem* item) {
       }
 
       case RootItem::Kind::Label: {
-        QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+        QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
         list = DatabaseQueries::customIdsOfMessagesFromLabel(database, item->toLabel());
         break;
       }
 
       case RootItem::Kind::ServiceRoot: {
-        QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+        QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
         list = DatabaseQueries::customIdsOfMessagesFromAccount(database, accountId());
         break;
       }
 
       case RootItem::Kind::Bin: {
-        QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+        QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
         list = DatabaseQueries::customIdsOfMessagesFromBin(database, accountId());
         break;
       }
 
       case RootItem::Kind::Feed: {
-        QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+        QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
         list = DatabaseQueries::customIdsOfMessagesFromFeed(database, item->customId(), accountId());
         break;
       }
 
       case RootItem::Kind::Important: {
-        QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+        QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
         list = DatabaseQueries::customIdsOfImportantMessages(database, accountId());
         break;
       }
 
+      case RootItem::Kind::Unread: {
+        QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
+
+        list = DatabaseQueries::customIdsOfUnreadMessages(database, accountId());
+        break;
+      }
+
       default:
         break;
     }
@@ -474,8 +565,8 @@ QStringList ServiceRoot::customIDSOfMessagesForItem(RootItem* item) {
   }
 }
 
-bool ServiceRoot::markFeedsReadUnread(QList<Feed*> items, RootItem::ReadStatus read) {
-  QSqlDatabase database = qApp->database()->connection(metaObject()->className());
+bool ServiceRoot::markFeedsReadUnread(const QList<Feed*>& items, RootItem::ReadStatus read) {
+  QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
   if (DatabaseQueries::markFeedsReadUnread(database, textualFeedIds(items), accountId(), read)) {
     getParentServiceRoot()->updateCounts(false);
@@ -494,26 +585,24 @@ QStringList ServiceRoot::textualFeedUrls(const QList<Feed*>& feeds) const {
   stringy_urls.reserve(feeds.size());
 
   for (const Feed* feed : feeds) {
-    stringy_urls.append(!feed->url().isEmpty() ? feed->url() : QL1S("no-url"));
+    stringy_urls.append(!feed->source().isEmpty() ? feed->source() : QSL("no-url"));
   }
 
   return stringy_urls;
 }
 
 QStringList ServiceRoot::textualFeedIds(const QList<Feed*>& feeds) const {
-  QStringList stringy_ids;
-
-  stringy_ids.reserve(feeds.size());
+  QStringList stringy_ids; stringy_ids.reserve(feeds.size());
 
   for (const Feed* feed : feeds) {
-    stringy_ids.append(QString("'%1'").arg(feed->customId()));
+    stringy_ids.append(QSL("'%1'").arg(feed->customId()));
   }
 
   return stringy_ids;
 }
 
 QStringList ServiceRoot::customIDsOfMessages(const QList<ImportanceChange>& changes) {
-  QStringList list;
+  QStringList list; list.reserve(changes.size());
 
   for (const auto& change : changes) {
     list.append(change.first.m_customId);
@@ -523,7 +612,7 @@ QStringList ServiceRoot::customIDsOfMessages(const QList<ImportanceChange>& chan
 }
 
 QStringList ServiceRoot::customIDsOfMessages(const QList<Message>& messages) {
-  QStringList list;
+  QStringList list; list.reserve(messages.size());
 
   for (const Message& message : messages) {
     list.append(message.m_customId);
@@ -548,25 +637,36 @@ void ServiceRoot::setAccountId(int account_id) {
 
 bool ServiceRoot::loadMessagesForItem(RootItem* item, MessagesModel* model) {
   if (item->kind() == RootItem::Kind::Bin) {
-    model->setFilter(QString("Messages.is_deleted = 1 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1")
+    model->setFilter(QSL("Messages.is_deleted = 1 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1")
                      .arg(QString::number(accountId())));
   }
   else if (item->kind() == RootItem::Kind::Important) {
-    model->setFilter(QString("Messages.is_important = 1 AND Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1")
+    model->setFilter(QSL("Messages.is_important = 1 AND Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1")
+                     .arg(QString::number(accountId())));
+  }
+  else if (item->kind() == RootItem::Kind::Unread) {
+    model->setFilter(QSL("Messages.is_read = 0 AND Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1")
                      .arg(QString::number(accountId())));
   }
   else if (item->kind() == RootItem::Kind::Label) {
     // Show messages with particular label.
-    model->setFilter(QString("Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1 AND "
-                             "(SELECT COUNT(*) FROM LabelsInMessages WHERE account_id = %1 AND message = Messages.custom_id AND label = '%2') > 0")
+    model->setFilter(QSL("Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1 AND "
+                         "(SELECT COUNT(*) FROM LabelsInMessages WHERE account_id = %1 AND message = Messages.custom_id AND label = '%2') > 0")
                      .arg(QString::number(accountId()), item->customId()));
   }
   else if (item->kind() == RootItem::Kind::Labels) {
     // Show messages with any label.
-    model->setFilter(QString("Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1 AND "
-                             "(SELECT COUNT(*) FROM LabelsInMessages WHERE account_id = %1 AND message = Messages.custom_id) > 0")
+    model->setFilter(QSL("Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1 AND "
+                         "(SELECT COUNT(*) FROM LabelsInMessages WHERE account_id = %1 AND message = Messages.custom_id) > 0")
                      .arg(QString::number(accountId())));
   }
+  else if (item->kind() == RootItem::Kind::ServiceRoot) {
+    model->setFilter(
+      QSL("Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %1").arg(
+        QString::number(accountId())));
+
+    qDebugNN << "Displaying messages from account:" << QUOTE_W_SPACE_DOT(accountId());
+  }
   else {
     QList<Feed*> children = item->getSubTreeFeeds();
     QString filter_clause = textualFeedIds(children).join(QSL(", "));
@@ -576,7 +676,7 @@ bool ServiceRoot::loadMessagesForItem(RootItem* item, MessagesModel* model) {
     }
 
     model->setFilter(
-      QString("Feeds.custom_id IN (%1) AND Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %2").arg(
+      QSL("Feeds.custom_id IN (%1) AND Messages.is_deleted = 0 AND Messages.is_pdeleted = 0 AND Messages.account_id = %2").arg(
         filter_clause,
         QString::
         number(accountId())));
@@ -647,7 +747,7 @@ bool ServiceRoot::onAfterSwitchMessageImportance(RootItem* selected_item, const
   Q_UNUSED(selected_item)
   Q_UNUSED(changes)
 
-  updateCounts(false);
+  updateCounts(true);
   itemChanged(getSubTree());
   return true;
 }
@@ -667,7 +767,9 @@ bool ServiceRoot::onAfterMessagesDelete(RootItem* selected_item, const QList<Mes
   return true;
 }
 
-bool ServiceRoot::onBeforeLabelMessageAssignmentChanged(const QList<Label*> labels, const QList<Message>& messages, bool assign) {
+bool ServiceRoot::onBeforeLabelMessageAssignmentChanged(const QList<Label*>& labels,
+                                                        const QList<Message>& messages,
+                                                        bool assign) {
   auto cache = dynamic_cast<CacheForServiceRoot*>(this);
 
   if (cache != nullptr) {
@@ -679,7 +781,9 @@ bool ServiceRoot::onBeforeLabelMessageAssignmentChanged(const QList<Label*> labe
   return true;
 }
 
-bool ServiceRoot::onAfterLabelMessageAssignmentChanged(const QList<Label*> labels, const QList<Message>& messages, bool assign) {
+bool ServiceRoot::onAfterLabelMessageAssignmentChanged(const QList<Label*>& labels,
+                                                       const QList<Message>& messages,
+                                                       bool assign) {
   Q_UNUSED(messages)
   Q_UNUSED(assign)
 
@@ -715,7 +819,7 @@ CacheForServiceRoot* ServiceRoot::toCache() const {
   return dynamic_cast<CacheForServiceRoot*>(const_cast<ServiceRoot*>(this));
 }
 
-void ServiceRoot::assembleFeeds(Assignment feeds) {
+void ServiceRoot::assembleFeeds(const Assignment& feeds) {
   QHash<int, Category*> categories = getHashedSubTreeCategories();
 
   for (const AssignmentItem& feed : feeds) {
@@ -733,24 +837,25 @@ void ServiceRoot::assembleFeeds(Assignment feeds) {
   }
 }
 
-void ServiceRoot::assembleCategories(Assignment categories) {
+void ServiceRoot::assembleCategories(const Assignment& categories) {
+  Assignment editable_categories = categories;
   QHash<int, RootItem*> assignments;
 
   assignments.insert(NO_PARENT_CATEGORY, this);
 
   // Add top-level categories.
-  while (!categories.isEmpty()) {
-    for (int i = 0; i < categories.size(); i++) {
-      if (assignments.contains(categories.at(i).first)) {
+  while (!editable_categories.isEmpty()) {
+    for (int i = 0; i < editable_categories.size(); i++) {
+      if (assignments.contains(editable_categories.at(i).first)) {
         // Parent category of this category is already added.
-        assignments.value(categories.at(i).first)->appendChild(categories.at(i).second);
+        assignments.value(editable_categories.at(i).first)->appendChild(editable_categories.at(i).second);
 
         // Now, added category can be parent for another categories, add it.
-        assignments.insert(categories.at(i).second->id(), categories.at(i).second);
+        assignments.insert(editable_categories.at(i).second->id(), editable_categories.at(i).second);
 
         // Remove the category from the list, because it was
         // added to the final collection.
-        categories.removeAt(i);
+        editable_categories.removeAt(i);
         i--;
       }
     }
@@ -764,3 +869,58 @@ ServiceRoot::LabelOperation operator|(ServiceRoot::LabelOperation lhs, ServiceRo
 ServiceRoot::LabelOperation operator&(ServiceRoot::LabelOperation lhs, ServiceRoot::LabelOperation rhs) {
   return static_cast<ServiceRoot::LabelOperation>(static_cast<char>(lhs) & static_cast<char>(rhs));
 }
+
+QPair<int, int> ServiceRoot::updateMessages(QList<Message>& messages, Feed* feed, bool force_update) {
+  QPair<int, int> updated_messages = { 0, 0 };
+
+  if (messages.isEmpty()) {
+    qDebugNN << "No messages to be updated/added in DB for feed"
+             << QUOTE_W_SPACE_DOT(feed->customId());
+    return updated_messages;
+  }
+
+  QList<RootItem*> items_to_update;
+  bool is_main_thread = QThread::currentThread() == qApp->thread();
+
+  qDebugNN << LOGSEC_CORE
+           << "Updating messages in DB. Main thread:"
+           << QUOTE_W_SPACE_DOT(is_main_thread);
+
+  bool ok = false;
+  QSqlDatabase database = is_main_thread ?
+                          qApp->database()->driver()->connection(metaObject()->className()) :
+                          qApp->database()->driver()->connection(QSL("feed_upd"));
+
+  updated_messages = DatabaseQueries::updateMessages(database, messages, feed, force_update, &ok);
+
+  if (updated_messages.first > 0 || updated_messages.second > 0) {
+    // Something was added or updated in the DB, update numbers.
+    feed->updateCounts(true);
+
+    if (recycleBin() != nullptr) {
+      recycleBin()->updateCounts(true);
+      items_to_update.append(recycleBin());
+    }
+
+    if (importantNode() != nullptr) {
+      importantNode()->updateCounts(true);
+      items_to_update.append(importantNode());
+    }
+
+    if (unreadNode() != nullptr) {
+      unreadNode()->updateCounts(true);
+      items_to_update.append(unreadNode());
+    }
+
+    if (labelsNode() != nullptr) {
+      labelsNode()->updateCounts(true);
+      items_to_update.append(labelsNode());
+    }
+  }
+
+  // Some messages were really added to DB, reload feed in model.
+  items_to_update.append(feed);
+  getParentServiceRoot()->itemChanged(items_to_update);
+
+  return updated_messages;
+}
