@@ -13,12 +13,13 @@
 class AtomParser : public FeedParser {
   public:
     explicit AtomParser(const QString& data);
-    virtual ~AtomParser();
+
+    QString atomNamespace() const;
 
   private:
     QDomNodeList messageElements();
     QString feedAuthor() const;
-    Message extractMessage(const QDomElement& msg_element, QDateTime current_time) const;
+    Message extractMessage(const QDomElement& msg_element, const QDateTime& current_time) const;
     QString messageAuthor(const QDomElement& msg_element) const;
 
   private:
