@@ -2,16 +2,19 @@
 
 #include "services/standard/gui/formeditstandardaccount.h"
 
+#include "database/databasefactory.h"
+#include "database/databasequeries.h"
 #include "miscellaneous/application.h"
-#include "miscellaneous/databasefactory.h"
-#include "miscellaneous/databasequeries.h"
 #include "miscellaneous/iconfactory.h"
+#include "services/standard/standardserviceentrypoint.h"
 #include "services/standard/standardserviceroot.h"
 
 FormEditStandardAccount::FormEditStandardAccount(QWidget* parent)
-  : FormAccountDetails(qApp->icons()->fromTheme(QSL("rssguard")), parent) {}
+  : FormAccountDetails(StandardServiceEntryPoint().icon(), parent) {}
 
 void FormEditStandardAccount::apply() {
-  applyInternal<StandardServiceRoot>();
+  FormAccountDetails::apply();
+
+  m_account->saveAccountDataToDatabase();
   accept();
 }
