@@ -6,6 +6,7 @@
 #include "services/abstract/gui/formfeeddetails.h"
 
 class StandardFeedDetails;
+class StandardServiceRoot;
 class AuthenticationDetails;
 class StandardFeed;
 
@@ -13,23 +14,24 @@ class FormStandardFeedDetails : public FormFeedDetails {
   Q_OBJECT
 
   public:
-    explicit FormStandardFeedDetails(ServiceRoot* service_root, QWidget* parent = nullptr);
-
-  public slots:
-    int addEditFeed(StandardFeed* input_feed, RootItem* parent_to_select, const QString& url = QString());
+    explicit FormStandardFeedDetails(ServiceRoot* service_root, RootItem* parent_to_select = nullptr,
+                                     const QString& url = QString(), QWidget* parent = nullptr);
 
   private slots:
     void guessFeed();
     void guessIconOnly();
+    void onTitleChanged(const QString& title);
 
     virtual void apply();
 
   private:
-    virtual void setEditableFeed(Feed* editable_feed);
+    virtual void loadFeedData();
 
   private:
     StandardFeedDetails* m_standardFeedDetails;
     AuthenticationDetails* m_authDetails;
+    RootItem* m_parentToSelect;
+    QString m_urlToProcess;
 };
 
 #endif // FORMSSFEEDDETAILS_H
