@@ -2,10 +2,14 @@
 
 #include "services/standard/gui/standardfeeddetails.h"
 
+#include "exceptions/applicationexception.h"
+#include "exceptions/networkexception.h"
+#include "exceptions/scriptexception.h"
 #include "gui/guiutilities.h"
 #include "miscellaneous/iconfactory.h"
 #include "network-web/networkfactory.h"
 #include "services/abstract/category.h"
+#include "services/standard/definitions.h"
 
 #include <QClipboard>
 #include <QFileDialog>
@@ -74,6 +78,7 @@ StandardFeedDetails::StandardFeedDetails(QWidget* parent) : QWidget(parent) {
   m_ui.m_txtSource->textEdit()->setFocus(Qt::FocusReason::TabFocusReason);
 
   // Set feed metadata fetch label.
+  m_ui.m_lblFetchMetadata->label()->setWordWrap(true);
   m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Information,
                                      tr("No metadata fetched so far."),
                                      tr("No metadata fetched so far."));
@@ -114,37 +119,36 @@ StandardFeedDetails::StandardFeedDetails(QWidget* parent) : QWidget(parent) {
 void StandardFeedDetails::guessIconOnly(StandardFeed::SourceType source_type, const QString& source,
                                         const QString& post_process_script, const QString& username,
                                         const QString& password, const QNetworkProxy& custom_proxy) {
-  bool result;
-  StandardFeed* metadata = StandardFeed::guessFeed(source_type,
-                                                   source,
-                                                   post_process_script,
-                                                   &result,
-                                                   username,
-                                                   password,
-                                                   custom_proxy);
+  try {
+    StandardFeed* metadata = StandardFeed::guessFeed(source_type,
+                                                     source,
+                                                     post_process_script,
+                                                     username,
+                                                     password,
+                                                     custom_proxy);
 
-  if (metadata != nullptr) {
     // Icon or whole feed was guessed.
     m_ui.m_btnIcon->setIcon(metadata->icon());
-
-    if (result) {
-      m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Ok,
-                                         tr("Icon fetched successfully."),
-                                         tr("Icon metadata fetched."));
-    }
-    else {
-      m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Warning,
-                                         tr("Icon metadata not fetched."),
-                                         tr("Icon metadata not fetched."));
-    }
+    m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Ok,
+                                       tr("Icon fetched successfully."),
+                                       tr("Icon metadata fetched."));
 
     // Remove temporary feed object.
-    delete metadata;
+    metadata->deleteLater();
   }
-  else {
-    // No feed guessed, even no icon available.
+  catch (const ScriptException& ex) {
     m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Error,
-                                       tr("No icon fetched."),
+                                       tr("Script failed: %1").arg(ex.message()),
+                                       tr("No icon fetched."));
+  }
+  catch (const NetworkException& ex) {
+    m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Error,
+                                       tr("Network error: %1").arg(ex.message()),
+                                       tr("No icon fetched."));
+  }
+  catch (const ApplicationException& ex) {
+    m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Error,
+                                       tr("Error: %1").arg(ex.message()),
                                        tr("No icon fetched."));
   }
 }
@@ -152,16 +156,14 @@ void StandardFeedDetails::guessIconOnly(StandardFeed::SourceType source_type, co
 void StandardFeedDetails::guessFeed(StandardFeed::SourceType source_type, const QString& source,
                                     const QString& post_process_script, const QString& username,
                                     const QString& password, const QNetworkProxy& custom_proxy) {
-  bool result;
-  StandardFeed* metadata = StandardFeed::guessFeed(source_type,
-                                                   source,
-                                                   post_process_script,
-                                                   &result,
-                                                   username,
-                                                   password,
-                                                   custom_proxy);
+  try {
+    StandardFeed* metadata = StandardFeed::guessFeed(source_type,
+                                                     source,
+                                                     post_process_script,
+                                                     username,
+                                                     password,
+                                                     custom_proxy);
 
-  if (metadata != nullptr) {
     // Icon or whole feed was guessed.
     m_ui.m_btnIcon->setIcon(metadata->icon());
     m_ui.m_txtTitle->lineEdit()->setText(metadata->title());
@@ -173,33 +175,36 @@ void StandardFeedDetails::guessFeed(StandardFeed::SourceType source_type, const
       m_ui.m_cmbEncoding->setCurrentIndex(encoding_index);
     }
     else {
-      m_ui.m_cmbEncoding->setCurrentIndex(m_ui.m_cmbEncoding->findText(DEFAULT_FEED_ENCODING, Qt::MatchFixedString));
+      m_ui.m_cmbEncoding->setCurrentIndex(m_ui.m_cmbEncoding->findText(QSL(DEFAULT_FEED_ENCODING),
+                                                                       Qt::MatchFlag::MatchFixedString));
     }
 
-    if (result) {
-      m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Ok,
-                                         tr("All metadata fetched successfully."),
-                                         tr("Feed and icon metadata fetched."));
-    }
-    else {
-      m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Warning,
-                                         tr("Feed or icon metadata not fetched."),
-                                         tr("Feed or icon metadata not fetched."));
-    }
+    m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Ok,
+                                       tr("All metadata fetched successfully."),
+                                       tr("Feed and icon metadata fetched."));
 
     // Remove temporary feed object.
-    delete metadata;
+    metadata->deleteLater();
   }
-  else {
-    // No feed guessed, even no icon available.
+  catch (const ScriptException& ex) {
     m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Error,
-                                       tr("No metadata fetched."),
+                                       tr("Script failed: %1").arg(ex.message()),
+                                       tr("No metadata fetched."));
+  }
+  catch (const NetworkException& ex) {
+    m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Error,
+                                       tr("Network error: %1").arg(ex.message()),
+                                       tr("No metadata fetched."));
+  }
+  catch (const ApplicationException& ex) {
+    m_ui.m_lblFetchMetadata->setStatus(WidgetWithStatus::StatusType::Error,
+                                       tr("Error: %1").arg(ex.message()),
                                        tr("No metadata fetched."));
   }
 }
 
 void StandardFeedDetails::onTitleChanged(const QString& new_title) {
-  if (new_title.simplified().size() >= MIN_CATEGORY_NAME_LENGTH) {
+  if (!new_title.simplified().isEmpty()) {
     m_ui.m_txtTitle->setStatus(LineEditWithStatus::StatusType::Ok, tr("Feed name is ok."));
   }
   else {
@@ -218,7 +223,7 @@ void StandardFeedDetails::onDescriptionChanged(const QString& new_description) {
 
 void StandardFeedDetails::onUrlChanged(const QString& new_url) {
   if (sourceType() == StandardFeed::SourceType::Url) {
-    if (QRegularExpression(URL_REGEXP).match(new_url).hasMatch()) {
+    if (QRegularExpression(QSL(URL_REGEXP)).match(new_url).hasMatch()) {
       m_ui.m_txtSource->setStatus(LineEditWithStatus::StatusType::Ok, tr("The URL is ok."));
     }
     else if (!new_url.simplified().isEmpty()) {
@@ -231,7 +236,7 @@ void StandardFeedDetails::onUrlChanged(const QString& new_url) {
     }
   }
   else if (sourceType() == StandardFeed::SourceType::Script) {
-    if (QRegularExpression(SCRIPT_SOURCE_TYPE_REGEXP).match(new_url).hasMatch()) {
+    if (QRegularExpression(QSL(SCRIPT_SOURCE_TYPE_REGEXP)).match(new_url).hasMatch()) {
       m_ui.m_txtSource->setStatus(LineEditWithStatus::StatusType::Ok, tr("The source is ok."));
     }
     else if (!new_url.simplified().isEmpty()) {
@@ -248,7 +253,7 @@ void StandardFeedDetails::onUrlChanged(const QString& new_url) {
 }
 
 void StandardFeedDetails::onPostProcessScriptChanged(const QString& new_pp) {
-  if (QRegularExpression(SCRIPT_SOURCE_TYPE_REGEXP).match(new_pp).hasMatch()) {
+  if (QRegularExpression(QSL(SCRIPT_SOURCE_TYPE_REGEXP)).match(new_pp).hasMatch()) {
     m_ui.m_txtPostProcessScript->setStatus(LineEditWithStatus::StatusType::Ok, tr("Command is ok."));
   }
   else if (!new_pp.simplified().isEmpty()) {
@@ -293,7 +298,7 @@ void StandardFeedDetails::prepareForNewFeed(RootItem* parent_to_select, const QS
   // Make sure that "default" icon is used as the default option for new
   // feed.
   m_actionUseDefaultIcon->trigger();
-  int default_encoding_index = m_ui.m_cmbEncoding->findText(DEFAULT_FEED_ENCODING);
+  int default_encoding_index = m_ui.m_cmbEncoding->findText(QSL(DEFAULT_FEED_ENCODING));
 
   if (default_encoding_index >= 0) {
     m_ui.m_cmbEncoding->setCurrentIndex(default_encoding_index);
@@ -310,6 +315,9 @@ void StandardFeedDetails::prepareForNewFeed(RootItem* parent_to_select, const QS
         m_ui.m_cmbParentCategory->setCurrentIndex(target_item);
       }
     }
+    else {
+      m_ui.m_cmbParentCategory->setCurrentIndex(0);
+    }
   }
 
   if (!url.isEmpty()) {
@@ -328,7 +336,7 @@ void StandardFeedDetails::setExistingFeed(StandardFeed* feed) {
   m_ui.m_txtTitle->lineEdit()->setText(feed->title());
   m_ui.m_txtDescription->lineEdit()->setText(feed->description());
   m_ui.m_btnIcon->setIcon(feed->icon());
-  m_ui.m_txtSource->textEdit()->setPlainText(feed->url());
+  m_ui.m_txtSource->textEdit()->setPlainText(feed->source());
   m_ui.m_txtPostProcessScript->textEdit()->setPlainText(feed->postProcessScript());
   m_ui.m_cmbType->setCurrentIndex(m_ui.m_cmbType->findData(QVariant::fromValue(int(feed->type()))));
   m_ui.m_cmbEncoding->setCurrentIndex(m_ui.m_cmbEncoding->findData(feed->encoding(),
