@@ -12,11 +12,10 @@
 class RssParser : public FeedParser {
   public:
     explicit RssParser(const QString& data);
-    virtual ~RssParser();
 
   private:
     QDomNodeList messageElements();
-    Message extractMessage(const QDomElement& msg_element, QDateTime current_time) const;
+    Message extractMessage(const QDomElement& msg_element, const QDateTime& current_time) const;
 };
 
 #endif // RSSPARSER_H
