@@ -28,12 +28,9 @@ class FeedsImportExportModel : public AccountCheckSortedModel {
     void importAsTxtURLPerLine(const QByteArray& data, bool fetch_metadata_online);
 
     Mode mode() const;
-    void setMode(const Mode& mode);
+    void setMode(Mode mode);
 
   signals:
-
-    // These signals are emitted when user selects some data
-    // to be imported/parsed into the model.
     void parsingStarted();
     void parsingProgress(int completed, int total);
     void parsingFinished(int count_failed, int count_succeeded, bool parsing_error);
