@@ -19,7 +19,7 @@ class FormEditTtRssAccount : public FormAccountDetails {
     virtual void apply();
 
   protected:
-    virtual void setEditableAccount(ServiceRoot* editable_account);
+    virtual void loadAccountData();
 
   private slots:
     void performTest();
