@@ -11,17 +11,20 @@ class AuthenticationDetails;
 
 class FormTtRssFeedDetails : public FormFeedDetails {
   public:
-    explicit FormTtRssFeedDetails(ServiceRoot* service_root, QWidget* parent = nullptr);
-
-  public slots:
-    int addFeed(RootItem* parent_to_select, const QString& url = QString());
+    explicit FormTtRssFeedDetails(ServiceRoot* service_root, RootItem* parent_to_select = nullptr,
+                                  const QString& url = QString(), QWidget* parent = nullptr);
 
   protected slots:
     virtual void apply();
 
+  private:
+    virtual void loadFeedData();
+
   private:
     TtRssFeedDetails* m_feedDetails;
     AuthenticationDetails* m_authDetails;
+    RootItem* m_parentToSelect;
+    QString m_urlToProcess;
 };
 
 #endif // FORMTTRSSFEEDDETAILS_H
